<?php

namespace Pinpoint\Conesso\Block;

use Magento\Framework\View\Element\Template\Context;
use Pinpoint\Conesso\Block\Helper\Font;
use Pinpoint\Conesso\Helper\Config;
use Pinpoint\Conesso\Helper\Data;
use Pinpoint\Conesso\Model\DateIntervalFactory;
use Pinpoint\Conesso\Model\SalesRule\ConessoCouponGenerator;

/**
 * Coupon block
 *
 * @api
 */
class Coupon extends \Magento\Framework\View\Element\Template
{
    /**
     * @var Data
     */
    public $helper;

    /**
     * @var ConessoCouponGenerator
     */
    private $conessoCouponGenerator;

    /**
     * @var DateIntervalFactory
     */
    private $dateIntervalFactory;

    /**
     * @var Font
     */
    private $font;

    /**
     * Coupon constructor.
     *
     * @param Context                  $context
     * @param Data                     $helper
     * @param ConessoCouponGenerator $conessoCouponGenerator
     * @param DateIntervalFactory      $dateIntervalFactory
     * @param Font                     $font
     * @param array                    $data
     */
    public function __construct(
        Context $context,
        Data $helper,
        ConessoCouponGenerator $conessoCouponGenerator,
        DateIntervalFactory $dateIntervalFactory,
        Font $font,
        array $data = []
    ) {
        $this->dateIntervalFactory = $dateIntervalFactory;
        $this->helper = $helper;
        $this->conessoCouponGenerator = $conessoCouponGenerator;
        $this->font = $font;
        parent::__construct($context, $data);
    }

    /**
     * Generates the coupon code based on the code id.
     *
     * @return boolean
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function generateCoupon()
    {
        $params = $this->getRequest()->getParams();
        //check for param code and id
        if (! isset($params['code']) ||
            ! $this->helper->isCodeValid($params['code'])
        ) {
            return false;
        }

        $priceRuleId = (int) $params['id'];
        $expireDate = false;

        if (isset($params['expire_days']) && is_numeric($params['expire_days']) && $params['expire_days'] > 0) {
            $days = (int) $params['expire_days'];
            $expireDate = $this->_localeDate->date()
                ->add($this->dateIntervalFactory->create(['interval_spec' => sprintf('P%sD', $days)]));
        }

        return $this->conessoCouponGenerator->generateCoupon($priceRuleId, $expireDate);
    }

    /**
     * @return array
     */
    public function getStyle()
    {
        return explode(
            ',',
            $this->helper->getWebsiteConfig(Config::XML_PATH_CONNECTOR_DYNAMIC_COUPON_STYLE)
        );
    }

    /**
     * Coupon color from config.
     *
     * @return mixed
     */
    public function getCouponColor()
    {
        return $this->helper->getWebsiteConfig(
            Config::XML_PATH_CONNECTOR_DYNAMIC_COUPON_COLOR
        );
    }

    /**
     * Coupon font size from config.
     *
     * @return integer|boolean
     */
    public function getFontSize()
    {
        return $this->helper->getWebsiteConfig(
            Config::XML_PATH_CONNECTOR_DYNAMIC_COUPON_FONT_SIZE
        );
    }

    /**
     * @return boolean|string
     */
    public function getHtmlFontFamily()
    {
        return $this->font->getEscapedFontFamilyForCoupon();
    }

    /**
     * Coupon background color from config.
     *
     * @return string|boolean
     */
    public function getBackgroundColor()
    {
        return $this->helper->getWebsiteConfig(
            Config::XML_PATH_CONNECTOR_DYNAMIC_COUPON_BG_COLOR
        );
    }
}
