<?php

namespace Pinpoint\Conesso\Controller\Adminhtml\Contact;

use Magento\Backend\App\Action\Context;
use Magento\Customer\Model\CustomerFactory;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\Stdlib\DateTime;
use Magento\Framework\View\Result\PageFactory;
use Magento\Newsletter\Model\SubscriberFactory;
use Magento\Store\Model\StoreManagerInterface;
use Pinpoint\Conesso\Helper\Data;
use Pinpoint\Conesso\Model\Connector\AccountFactory;
use Pinpoint\Conesso\Model\ContactFactory;
use Pinpoint\Conesso\Model\OrderFactory;
use Pinpoint\Conesso\Model\ResourceModel\Contact;
use Pinpoint\Conesso\Model\ResourceModel\Contact\CollectionFactory;
use Pinpoint\Conesso\Model\ResourceModel\Order;
use Pinpoint\Conesso\Model\Sync\SetsSyncFromTime;

class Synccontacts extends \Magento\Backend\App\Action
{
    use SetsSyncFromTime;

    //customer
    const XML_PATH_LOSTBASKET_CUSTOMER_ENABLED_1 = 'conesso_abandoned_carts/customers/enabled_1';
    const XML_PATH_LOSTBASKET_CUSTOMER_ENABLED_2 = 'conesso_abandoned_carts/customers/enabled_2';
    const XML_PATH_LOSTBASKET_CUSTOMER_ENABLED_3 = 'conesso_abandoned_carts/customers/enabled_3';
    const XML_PATH_LOSTBASKET_CUSTOMER_INTERVAL_1 = 'conesso_abandoned_carts/customers/send_after_1';
    const XML_PATH_LOSTBASKET_CUSTOMER_INTERVAL_2 = 'conesso_abandoned_carts/customers/send_after_2';
    const XML_PATH_LOSTBASKET_CUSTOMER_INTERVAL_3 = 'conesso_abandoned_carts/customers/send_after_3';
    const XML_PATH_LOSTBASKET_CUSTOMER_CAMPAIGN_1 = 'conesso_abandoned_carts/customers/campaign_1';
    const XML_PATH_LOSTBASKET_CUSTOMER_CAMPAIGN_2 = 'conesso_abandoned_carts/customers/campaign_2';
    const XML_PATH_LOSTBASKET_CUSTOMER_CAMPAIGN_3 = 'conesso_abandoned_carts/customers/campaign_3';

    //guest
    const XML_PATH_LOSTBASKET_GUEST_ENABLED_1 = 'conesso_abandoned_carts/guests/enabled_1';
    const XML_PATH_LOSTBASKET_GUEST_ENABLED_2 = 'conesso_abandoned_carts/guests/enabled_2';
    const XML_PATH_LOSTBASKET_GUEST_ENABLED_3 = 'conesso_abandoned_carts/guests/enabled_3';
    const XML_PATH_LOSTBASKET_GUEST_INTERVAL_1 = 'conesso_abandoned_carts/guests/send_after_1';
    const XML_PATH_LOSTBASKET_GUEST_INTERVAL_2 = 'conesso_abandoned_carts/guests/send_after_2';
    const XML_PATH_LOSTBASKET_GUEST_INTERVAL_3 = 'conesso_abandoned_carts/guests/send_after_3';
    const XML_PATH_LOSTBASKET_GUEST_CAMPAIGN_1 = 'conesso_abandoned_carts/guests/campaign_1';
    const XML_PATH_LOSTBASKET_GUEST_CAMPAIGN_2 = 'conesso_abandoned_carts/guests/campaign_2';
    const XML_PATH_LOSTBASKET_GUEST_CAMPAIGN_3 = 'conesso_abandoned_carts/guests/campaign_3';

    const CUSTOMER_LOST_BASKET_ONE = 1;
    const CUSTOMER_LOST_BASKET_TWO = 2;
    const CUSTOMER_LOST_BASKET_THREE = 3;

    const GUEST_LOST_BASKET_ONE = 1;
    const GUEST_LOST_BASKET_TWO = 2;
    const GUEST_LOST_BASKET_THREE = 3;

    const STATUS_PENDING = 'PendingOptIn';
    const STATUS_CONFIRMED = 'Confirmed';
    const STATUS_SENT = 'Sent';
    const STATUS_EXPIRED = 'Expired';
    /**
     * @var CollectionFactory
     */
    private $contactCollectionFactory;

    /**
     * @var integer
     */
    private $countCustomers = 0;

    /**
     * @var integer
     */
    private $importedContacts = 0;

    /**
     * @var mixed
     */
    private $start;

    /**
     * @var Data
     */
    private $helper;

    /**
     * @var JsonFactory
     */
    private $resultJsonFactory;

    /**
     * Authorization level of a basic admin session
     *
     * @see _isAllowed()
     */
    const ADMIN_RESOURCE = 'Pinpoint_Conesso::contact';

    /**
     * @var PageFactory
     */
    private $resultPageFactory;

    /**
     * @var Contact
     */
    public $contactResource;

    /**
     * @var ContactFactory
     */
    public $contactFactory;

    /**
     * @var CustomerFactory
     */
    protected $customerFactory;

    /**
     * @var SubscriberFactory
     */
    private $subscriberFactory;

    /**
     * @var DateTime
     */
    private $dateTime;

    /**
     * @var \Magento\Sales\Model\ResourceModel\Order\CollectionFactory
     */
    protected $orderCollectionFactory;

    /**
     * @var OrderFactory
     */
    private $emailOrderFactory;

    /**
     * @var Order
     */
    private $orderResource;

    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * Global number of orders.
     *
     * @var array
     */
    public $countOrders = [
        'orders' => 0,
        'single_sync' => 0,
        'pending' => 0,
        'modified' => 0,
    ];

    /**
     * @var Account
     */
    private $account;

    /**
     * @var OrderFactory
     */
    private $orderFactory;

    /**
     * @var array
     */
    public $guests = [];

    /**
     * @var array
     */
    private $orderIds = [];

    /**
     * @var \Pinpoint\Conesso\Model\Connector\OrderFactory
     */
    private $connectorOrderFactory;

    /**
     * @var string
     */
    private $apiUsername;

    /**
     * @var string
     */
    private $apiPassword;

    /**
     * @var AccountFactory
     */
    private $accountFactory;

    protected $success;

    protected $messages = [];

    protected $contactsSynced = 0;

    protected $ordersSynced = 0;

    /**
     * @var \Pinpoint\Conesso\Model\ResourceModel\Order\CollectionFactory
     */
    private $orderCollection;

    /**
     * @var \Pinpoint\Conesso\Model\RulesFactory
     */
    private $rulesFactory;

    /**
     * @var \Pinpoint\Conesso\Model\DateIntervalFactory
     */
    private $dateIntervalFactory;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    private $scopeConfig;

    /**
     * @var \Pinpoint\Conesso\Model\AbandonedFactory
     */
    public $abandonedFactory;

    /**
     * @var \Pinpoint\Conesso\Model\ResourceModel\Abandoned
     */
    private $abandonedResource;

    /**
     * @var \Pinpoint\Conesso\Model\ResourceModel\Abandoned\CollectionFactory
     */
    public $abandonedCollectionFactory;

    /**
     * @var \Magento\Quote\Model\ResourceModel\Quote\CollectionFactory
     */
    public $quoteCollectionFactory;

    /**
     * @var \Pinpoint\Conesso\Model\AbandonedCart\CartInsight\Data
     */
    private $cartInsight;

    protected $cartsSynced = 0;

    /**
     * @param Context                                                    $context
     * @param JsonFactory                                                $resultJsonFactory
     * @param PageFactory                                                $resultPageFactory
     * @param Data                                                       $helper
     * @param CollectionFactory                                          $contactCollectionFactory
     * @param Contact                                                    $contactResource
     * @param ContactFactory                                             $contactFactory
     * @param SubscriberFactory                                          $subscriberFactory
     * @param CustomerFactory                                            $customerFactory
     * @param DateTime                                                   $dateTime
     * @param \Magento\Sales\Model\ResourceModel\Order\CollectionFactory $orderCollectionFactory
     * @param OrderFactory                                               $emailOrderFactory
     * @param Order                                                      $orderResource
     * @param StoreManagerInterface                                      $storeManagerInterface
     * @param OrderFactory                                               $orderFactory
     * @param \Pinpoint\Conesso\Model\Connector\OrderFactory           $connectorOrderFactory
     * @param AccountFactory                                             $accountFactory
     * @param Order\CollectionFactory                                    $collectionFactory
     * @param \Pinpoint\Conesso\Model\RulesFactory                     $rulesFactory
     * @param \Pinpoint\Conesso\Model\DateIntervalFactory              $dateIntervalFactory
     * @param \Pinpoint\Conesso\Model\AbandonedFactory                 $abandonedFactory
     * @param \Pinpoint\Conesso\Model\ResourceModel\Abandoned          $abandonedResource
     * @param \Magento\Quote\Model\ResourceModel\Quote\CollectionFactory $quoteCollectionFactory
     * @param \Pinpoint\Conesso\Model\AbandonedCart\CartInsight\Data   $cartInsight
     */
    public function __construct(
        Context $context,
        JsonFactory $resultJsonFactory,
        PageFactory $resultPageFactory,
        Data $helper,
        CollectionFactory $contactCollectionFactory,
        Contact $contactResource,
        ContactFactory $contactFactory,
        SubscriberFactory $subscriberFactory,
        CustomerFactory $customerFactory,
        DateTime $dateTime,
        \Magento\Sales\Model\ResourceModel\Order\CollectionFactory $orderCollectionFactory,
        OrderFactory $emailOrderFactory,
        Order $orderResource,
        StoreManagerInterface $storeManagerInterface,
        OrderFactory $orderFactory,
        \Pinpoint\Conesso\Model\Connector\OrderFactory $connectorOrderFactory,
        AccountFactory $accountFactory,
        \Pinpoint\Conesso\Model\ResourceModel\Order\CollectionFactory $collectionFactory,
        \Pinpoint\Conesso\Model\RulesFactory $rulesFactory,
        \Pinpoint\Conesso\Model\DateIntervalFactory $dateIntervalFactory,
        \Pinpoint\Conesso\Model\AbandonedFactory $abandonedFactory,
        \Pinpoint\Conesso\Model\ResourceModel\Abandoned $abandonedResource,
        \Magento\Quote\Model\ResourceModel\Quote\CollectionFactory $quoteCollectionFactory,
        \Pinpoint\Conesso\Model\AbandonedCart\CartInsight\Data $cartInsight
    ) {
        parent::__construct($context);
        $this->resultPageFactory = $resultPageFactory;
        $this->resultJsonFactory = $resultJsonFactory;
        $this->helper = $helper;
        $this->contactCollectionFactory = $contactCollectionFactory;
        $this->contactResource = $contactResource;
        $this->contactFactory   = $contactFactory;
        $this->customerFactory = $customerFactory;
        $this->subscriberFactory = $subscriberFactory;
        $this->dateTime = $dateTime;
        $this->orderCollectionFactory = $orderCollectionFactory;
        $this->emailOrderFactory      = $emailOrderFactory;
        $this->orderResource = $orderResource;
        $this->storeManager = $storeManagerInterface;
        $this->orderFactory          = $orderFactory;
        $this->connectorOrderFactory = $connectorOrderFactory;
        $this->accountFactory        = $accountFactory;
        $this->account = $this->accountFactory->create();
        $this->orderCollection = $collectionFactory;
        $this->rulesFactory = $rulesFactory;
        $this->dateIntervalFactory = $dateIntervalFactory;
        $this->scopeConfig = $this->helper->getScopeConfig();
        $this->abandonedFactory = $abandonedFactory;
        $this->abandonedResource = $abandonedResource;
        $this->abandonedCollectionFactory = $abandonedFactory->create()->abandonedCollectionFactory;
        $this->quoteCollectionFactory = $quoteCollectionFactory;
        $this->cartInsight = $cartInsight;
    }

    /**
     * Index action.
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $this->success = true;

        //starting time for sync
        $this->start = microtime(true);

        //export bulk contacts
        $this->importContacts();
        $this->importOrders();
        $this->importCarts();

        if ($this->success) {

            foreach ($this->helper->getWebsites() as $website) {

                $apiEnabled = $this->helper->isEnabled($website);
                $customerSyncEnabled = $this->helper->isCustomerSyncEnabled($website);
                $orderSyncEnabled = $this->helper->isOrderSyncEnabled($website);
                $cartSyncEnabled = $this->helper->isOrderSyncEnabled($website);

                if ($apiEnabled && $customerSyncEnabled) {
                    //start log
                    $contactsUpdated = $this->syncContacts($website);
                    if ($contactsUpdated) {
                        $this->messages[] = 'Success - Contacts Synced: '.$this->contactsSynced;
                    }
                }

                if ($apiEnabled && $orderSyncEnabled) {
                    //start log
                    $ordersSynced = $this->syncOrders($website);
                    if ($ordersSynced) {
                        $this->messages[] = 'Success - Orders Synced: '.$this->ordersSynced;
                        $this->ordersSynced = 0;
                    }
                }

                if ($apiEnabled && $cartSyncEnabled) {
                    //start log
                    $cartsSynced = $this->syncCarts($website);
                    if ($cartsSynced) {
                        $this->messages[] = 'Success - Carts Synced: '.$this->ordersSynced;
                        $this->ordersSynced = 0;
                    }
                }
            }
        }

        $result['success'] = $this->success;
        $result['messages'] = $this->messages;

        /** @var \Magento\Framework\Controller\Result\Json $resultJson */
        $resultJson = $this->resultJsonFactory->create();
        return $resultJson->setData($result);
    }



    protected function importContacts()
    {

        $collection = $this->customerFactory->create()->getCollection()
            ->addAttributeToSelect("*")
            ->addFieldToFilter(
                'entity_id',
                ['nin' => $this->importedContacts()]
            )
            ->load();
        $this->messages[] = 'Imported Contacts: '.$collection->getSize();
        foreach ($collection as $customer) {
            $customerId = $customer->getId();
            $customerFirstname = $customer->getFirstname();
            $websiteId  = $customer->getWebsiteId();
            $storeId = $customer->getStoreId();
            $email = $customer->getEmail();
            $subscriber = $this->subscriberFactory->create()
                ->loadByCustomerId($customerId);
            $isSubscribed = $subscriber->isSubscribed();

            //check if enabled
            if (!$this->helper->isEnabled($websiteId)) {
                return $this;
            }

            $emailBefore = '';

            $isContactExist = $this->contactFactory->create()->loadByCustomerId($customerId);
            if ($isContactExist->getId()) {
                $emailBefore = $isContactExist->getEmail();
            }

            if ($emailBefore !== $email) {
                $contactModel = $this->contactFactory->create()
                    ->loadByCustomerEmail($email, $websiteId);
                $contactModel
                    ->setEmail($email)
                    ->setEmailImported(\Pinpoint\Conesso\Model\Contact::EMAIL_CONTACT_NOT_IMPORTED)
                    ->setStoreId($storeId)
                    ->setCustomerId($customerId);
                if ($isSubscribed) {
                    $contactModel->setLastSubscribedAt($this->dateTime->formatDate(true))
                        ->setIsSubscriber(1)
                        ->setSubscriberStatus(1);
                }
                $this->contactResource->save($contactModel);
                $this->importedContacts++;
            }
        }

        return $this;
    }

    /**
     * @return array
     */
    protected function importedContacts()
    {
        $importedIDs = [0];
        $collection = $this->contactFactory->create()->getCollection();
        if ($collection->getSize() > 0) {
            foreach ($collection->getItems() as $item) {
                $importedIDs[] = $item->getCustomerId();
            }
        }
        return $importedIDs;
    }

    protected function importOrders()
    {

        $collection = $this->orderCollectionFactory->create()
            ->addFieldToFilter(
                'entity_id',
                ['nin' => $this->importedOrders()]
            );
        $this->messages[] = 'Imported Orders: '.$collection->getSize();
        foreach ($collection as $order) {
            $status         = $order->getStatus();
            $storeId        = $order->getStoreId();
            $customerEmail  = $order->getCustomerEmail();
            $store      = $this->storeManager->getStore($storeId);
            $storeName  = $store->getName();
            $websiteId  = $store->getWebsiteId();
            $emailOrder = $this->emailOrderFactory->create()
                ->loadByOrderId($order->getEntityId(), $order->getQuoteId());
            //reimport email order
            $emailOrder->setUpdatedAt($order->getUpdatedAt())
                ->setCreatedAt($order->getUpdatedAt())
                ->setStoreId($store->getId())
                ->setOrderStatus($status);

            if ($emailOrder->getConessoImported() != \Pinpoint\Conesso\Model\Contact::EMAIL_CONTACT_IMPORTED) {
                $emailOrder->setConessoImported(null);
            }

            $isEnabled = $this->helper->isStoreEnabled($store->getId());

            //api not enabled, stop emulation and exit
            if (!$isEnabled) {
                return $this;
            }

            // set back the current store
            $this->orderResource->save($emailOrder);
        }

        return $this;
    }

    /**
     * @return array
     */
    protected function importedOrders()
    {
        $importedIDs = [];
        $collection = $this->emailOrderFactory->create()->getCollection();
        if ($collection->getSize() > 0) {
            foreach ($collection->getItems() as $item) {
                $importedIDs[] = $item->getOrderId();
            }
        }
        return $importedIDs;
    }


    protected function syncContacts($website)
    {

        $syncLimit = $this->helper->getSyncLimit($website);

        $onlySubscribers = $this->helper->isOnlySubscribersForContactSync($website->getId());
        $contacts = $this->contactCollectionFactory->create();
        $contacts = ($onlySubscribers) ? $contacts->getContactsToImportByWebsite($website->getId(), $syncLimit, true) :
            $contacts->getContactsToImportByWebsite($website->getId(), $syncLimit);

        $message = '(Website '.$website->getId().') Syncing Contacts: '.$contacts->getSize();

        // no contacts found
        if (!$contacts->getSize()) {
            $message .= ' - Skipped';
            $this->messages[] = $message;
            return 0;
        }

        // Import Contacts
        foreach ($contacts->getItems() as $contact) {
            if ($contact->getData()) {

                $syncedInfo = $this->helper->syncContactWithConesso($contact);
                if ($syncedInfo['success'] === true) {
                    $contactInfo = $contact->getData();
                    $contact = $this->contactFactory->create();
                    $contact->load($contactInfo['conesso_contact_id']);
                    $contact->setConessoImported(1);
                    $contact->setSubscriberImported(1);
                    $contact->setContactId($syncedInfo['contact_id']);
                    $contact->save();
                    $this->countCustomers++;
                }
            }
        }

        $this->contactsSynced++;
        return true;
    }

    protected function syncOrders($website)
    {

        // Initialise a return hash containing results of our sync attempt
        $this->searchWebsiteAccounts($website);

        $orders = $this->account->getOrders();

        $ordersForSingleSync = $this->account->getOrdersForSingleSync();
        $numOrders = count($orders);
        $numOrdersForSingleSync = count($ordersForSingleSync);
        $totalOrders = $numOrders + $numOrdersForSingleSync;
        $message = '(Website '.$website->getId().') Syncing Orders: '.$totalOrders;

        $this->countOrders['orders'] += $numOrders;
        $this->countOrders['single_sync'] += $numOrdersForSingleSync;

        //create bulk
        if ($numOrders) {
            $syncedInfo = $this->helper->syncBulkOrdersWithConesso($orders);
            if ($syncedInfo['success'] == true) {
                //$this->orderResource->setImported($this->orderIds);
                $this->ordersSynced += $numOrders;
                $message .= ' Bulk Synced: '.$numOrders;
            } else {

            }

            $this->helper->log('--------- Order sync ---------- : ' . $numOrders);
        }
        //create single
        if ($numOrdersForSingleSync) {
            echo 'SYNC SINGLE ORDERS';
            $syncedInfo = $this->helper->syncBulkOrdersWithConesso($orders);
            if ($syncedInfo['success'] == true) {
                $this->ordersSynced += $numOrdersForSingleSync;
                $message .= ' SingleSync Synced: '.$numOrders;
                //$this->orderResource->setImported($this->orderIds);
            }
        }

        $this->messages[] = $message;
        //mark the orders as imported
        //$this->orderResource->setImported($this->orderIds);
        //unset($this->account);
        /**
         * Add guests to contact table.
         */
        if (!empty($this->guests)) {
            $orderEmails = array_keys($this->guests);
            $guestsEmailFound = $this->contactCollectionFactory->create()
                ->addFieldToFilter('email', ['in' => $orderEmails])
                ->getColumnValues('email');
            //remove the contacts that already exists
            foreach ($guestsEmailFound as $email) {
                unset($this->guests[strtolower($email)]);
            }

            //insert new guests contacts
            $this->contactResource->insertGuests($this->guests);
            //mark the existing contacts with is guest in bulk
            $this->contactResource->updateContactsAsGuests($guestsEmailFound);
        }
        unset($this->account);

        return true;
    }

    public function searchWebsiteAccounts($website)
    {
        $this->account = $this->accountFactory->create();
        $this->apiUsername = $this->helper->getApiUsername($website);
        $this->apiPassword = $this->helper->getApiPassword($website);
        // limit for orders included to sync
        $limit = $this->helper->getWebsiteConfig(
            \Pinpoint\Conesso\Helper\Config::XML_PATH_CONNECTOR_TRANSACTIONAL_DATA_SYNC_LIMIT,
            $website
        );

        $pendingOrders = $this->getPendingConnectorOrders($website, $limit);
        if (!empty($pendingOrders)) {
            $this->countOrders['pending'] += count($pendingOrders);
            $this->account->setOrders($pendingOrders);
        }
        $this->account->setWebsites($website->getId());

        $modifiedOrders = $this->getModifiedOrders($website, $limit);
        if (! empty($modifiedOrders)) {
            $this->countOrders['modified'] += count($modifiedOrders);
            $this->account->setOrdersForSingleSync($modifiedOrders);
        }
    }

    /**
     * @param \Magento\Store\Api\Data\WebsiteInterface $website
     * @param integer                                  $limit
     *
     * @return array
     */
    public function getPendingConnectorOrders($website, $limit = 100)
    {
        $orders = [];
        $storeIds = $website->getStoreIds();
        $orderModel = $this->orderFactory->create();
        //get order statuses set in configuration section
        $orderStatuses = $this->helper->getConfigSelectedStatus($website);

        //no active store for website
        if (empty($storeIds) || empty($orderStatuses)) {
            return [];
        }

        //pending order from conesso_order
        $orderCollection = $orderModel->getOrdersToImport($storeIds, $limit, $orderStatuses);

        //no orders found
        if (! $orderCollection->getSize()) {
            return $orders;
        }

        $orders = $this->mapOrderData($orderCollection, $orderModel, $orders);

        return $orders;
    }

    /**
     * @param \Dotdigitalgroup\Email\Model\ResourceModel\Order\Collection $orderCollection
     * @param \Dotdigitalgroup\Email\Model\Order                          $orderModel
     * @param array                                                       $orders
     *
     * @return array
     */
    protected function mapOrderData($orderCollection, $orderModel, $orders)
    {
        $orderIds = $orderCollection->getColumnValues('order_id');

        //get the order collection
        $salesOrderCollection = $orderModel->getSalesOrdersWithIds($orderIds);

        foreach ($salesOrderCollection as $order) {
            if ($order->getId()) {
                $storeId = $order->getStoreId();
                $websiteId = $this->helper->storeManager->getStore($storeId)->getWebsiteId();

                /**
                 * Add guest to array to add to contacts table.
                 */
                if ($order->getCustomerIsGuest() && $order->getCustomerEmail()) {
                    $email = $order->getCustomerEmail();
                    $this->guests[strtolower($email)] = [
                        'email' => $email,
                        'website_id' => $websiteId,
                        'store_id' => $storeId,
                        'is_guest' => 1
                    ];
                }

                $connectorOrder = $this->connectorOrderFactory->create();
                $connectorOrder->setOrderData($order);
                $orders[] = $connectorOrder;
            }

            $this->orderIds[] = $order->getId();
        }

        return $orders;
    }

    /**
     * @param \Magento\Store\Api\Data\WebsiteInterface $website
     * @param integer                                  $limit
     *
     * @return array
     */
    protected function getModifiedOrders($website, $limit)
    {
        $orders =  [];
        $storeIds = $website->getStoreIds();
        /** @var \Pinpoint\Conesso\Model\Order $orderModel */
        $orderModel = $this->orderFactory->create();
        //get order statuses set in configuration section
        $orderStatuses = $this->helper->getConfigSelectedStatus($website);

        //no active store for website
        if (empty($storeIds) || empty($orderStatuses)) {
            return [];
        }

        //pending order from email_order
        $orderCollection = $orderModel->getModifiedOrdersToImport($storeIds, $limit, $orderStatuses);

        //no orders found
        if (! $orderCollection->getSize()) {
            return $orders;
        }

        $orders = $this->mapOrderData($orderCollection, $orderModel, $orders);

        return $orders;
    }

    public function importCarts()
    {
        $result = [];
        $stores = $this->helper->getStores();

        foreach ($stores as $store) {
            $storeId = $store->getId();
            $websiteId = $store->getWebsiteId();

            $result = $this->processAbandonedCartsForCustomers($storeId, $websiteId, $result);
            //$result = $this->processAbandonedCartsForGuests($storeId, $websiteId, $result);
        }
    }

    /**
     * Process abandoned carts for customer
     *
     * @param integer $storeId
     * @param integer $websiteId
     * @param array   $result
     *
     * @return array
     */
    private function processAbandonedCartsForCustomers($storeId, $websiteId, $result)
    {

        $secondCustomerEnabled = $this->isLostBasketCustomerEnabled(self::CUSTOMER_LOST_BASKET_TWO, $storeId);
        $thirdCustomerEnabled = $this->isLostBasketCustomerEnabled(self::CUSTOMER_LOST_BASKET_THREE, $storeId);

        //first customer
        if ($this->isLostBasketCustomerEnabled(self::CUSTOMER_LOST_BASKET_ONE, $storeId) ||
            $secondCustomerEnabled ||
            $thirdCustomerEnabled
        ) {
            $result[$storeId]['firstCustomer'] = $this->processCustomerFirstAbandonedCart($storeId);
        }

        //second customer
        if ($secondCustomerEnabled) {
            $result[$storeId]['secondCustomer'] = $this->processExistingAbandonedCart(
                $storeId,
                $websiteId,
                self::CUSTOMER_LOST_BASKET_TWO
            );
        }

        //third customer
        if ($thirdCustomerEnabled) {
            $result[$storeId]['thirdCustomer'] = $this->processExistingAbandonedCart(
                $storeId,
                $websiteId,
                self::CUSTOMER_LOST_BASKET_THREE
            );
        }


        return $result;
    }

    /**
     * Process abandoned carts for guests
     *
     * @param integer $storeId
     * @param integer $websiteId
     * @param array   $result
     *
     * @return array
     */
    private function processAbandonedCartsForGuests($storeId, $websiteId, $result)
    {

        $secondGuestEnabled = $this->isLostBasketGuestEnabled(self::GUEST_LOST_BASKET_TWO, $storeId);
        $thirdGuestEnabled = $this->isLostBasketGuestEnabled(self::GUEST_LOST_BASKET_THREE, $storeId);

        //first guest
        if ($this->isLostBasketGuestEnabled(self::GUEST_LOST_BASKET_ONE, $storeId) ||
            $secondGuestEnabled ||
            $thirdGuestEnabled
        ) {
            $result[$storeId]['firstGuest'] = $this->processGuestFirstAbandonedCart($storeId);
        }
        //second guest
        if ($secondGuestEnabled) {
            $result[$storeId]['secondGuest'] = $this->processExistingAbandonedCart(
                $storeId,
                $websiteId,
                self::GUEST_LOST_BASKET_TWO,
                true
            );
        }
        //third guest
        if ($thirdGuestEnabled) {
            $result[$storeId]['thirdGuest'] = $this->processExistingAbandonedCart(
                $storeId,
                $websiteId,
                self::GUEST_LOST_BASKET_THREE,
                true
            );
        }

        return $result;
    }

    /**
     * @param integer $storeId
     * @return integer|string
     */
    private function processCustomerFirstAbandonedCart($storeId)
    {
        $abandonedNum = 1;

        $fromTime = $this->getSyncFromTime($this->getInterval($storeId, $abandonedNum));
        $toTime = clone $fromTime;
        $fromTime->sub($this->dateIntervalFactory->create(['interval_spec' => 'PT5M']));
        $fromDate = $fromTime->format('Y-m-d H:i:s');
        $toDate = $toTime->format('Y-m-d H:i:s');

        //active quotes
        $quoteCollection = $this->getStoreQuotes($fromDate, $toDate, false, $storeId);

        //found abandoned carts
        if ($quoteCollection->getSize()) {
            $this->helper->log('Customer AC 1 ' . $fromDate . ' - ' . $toDate);
        }

        $result = $this->createCustomerFirstAbandonedCart($quoteCollection, $storeId);
        $result += $this->processConfirmedCustomerAbandonedCart($storeId);

        return $result;
    }

    /**
     * @param integer $storeId
     * @return integer
     */
    private function processGuestFirstAbandonedCart($storeId)
    {
        $abandonedNum = 1;

        $fromTime = $this->getSyncFromTime($this->getSendAfterIntervalForGuest($storeId, $abandonedNum));
        $toTime = clone $fromTime;
        $fromTime->sub($this->dateIntervalFactory->create(['interval_spec' => 'PT5M']));

        //format time
        $fromDate   = $fromTime->format('Y-m-d H:i:s');
        $toDate     = $toTime->format('Y-m-d H:i:s');

        $quoteCollection = $this->getStoreQuotes($fromDate, $toDate, true, $storeId);

        if ($quoteCollection->getSize()) {
            $this->helper->log('Guest AC 1 ' . $fromDate . ' - ' . $toDate);
        }

        $guestCampaignId = $this->getLostBasketGuestCampaignId($abandonedNum, $storeId);
        $result = $this->createGuestFirstAbandonedCart($quoteCollection, $storeId, $guestCampaignId);
        $result += $this->processConfirmedGuestAbandonedCart($storeId, $guestCampaignId);


        return $result;
    }

    /**
     * @param string|null $from
     * @param string|null $to
     * @param boolean     $guest
     * @param integer     $storeId
     * @return \Magento\Quote\Model\ResourceModel\Quote\Collection|\Magento\Sales\Model\ResourceModel\Order\Collection
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getStoreQuotes($from = null, $to = null, $guest = false, $storeId = 0)
    {
        $updated = [
            'from' => $from,
            'to' => $to,
            'date' => true,
        ];
        $salesCollection = $this->orderCollection->create()
            ->getStoreQuotes($storeId, $updated, $guest);

        //process rules on collection
        $ruleModel = $this->rulesFactory->create();
        $websiteId = $this->helper->storeManager->getStore($storeId)
            ->getWebsiteId();

        $salesCollection = $ruleModel->process(
            $salesCollection,
            \Pinpoint\Conesso\Model\Rules::ABANDONED,
            $websiteId
        );

        return $salesCollection;
    }

    /**
     * @param $quoteCollection
     * @param $storeId
     * @param $campaignId
     *
     * @return integer
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    private function createCustomerFirstAbandonedCart($quoteCollection, $storeId)
    {
        $result = 0;
        foreach ($quoteCollection as $quote) {
            $websiteId = $this->helper->storeManager->getStore($storeId)->getWebsiteId();
            if (!$this->updateDataFieldAndCreateAc($quote, $websiteId, $storeId)) {
                continue;
            }
            $result++;
        }
        return $result;
    }

    /**
     * @param \Magento\Quote\Model\Quote $quote
     * @param integer                    $websiteId
     * @param integer                    $storeId
     *
     * @return boolean
     */
    private function updateDataFieldAndCreateAc($quote, $websiteId, $storeId)
    {
        $this->cartInsight->send($quote, $storeId);

        $quoteId = $quote->getId();
        $items = $quote->getAllItems();
        $email = $quote->getCustomerEmail();
        $itemIds = $this->getQuoteItemIds($items);
        $abandonedModel = $this->abandonedFactory->create()
            ->loadByQuoteId($quoteId);

        $this->createAbandonedCart($abandonedModel, $quote, $itemIds, self::STATUS_PENDING);

        if ($this->abandonedCartAlreadyExists($abandonedModel) &&
            $this->shouldNotSendACAgain($abandonedModel, $quote) &&
            $this->isNotAConfirmedContact($abandonedModel)
        ) {
            if ($this->shouldDeleteAbandonedCart($quote)) {
                $this->deleteAbandonedCart($abandonedModel);
            }
            return false;
        } else {
            if ($mostExpensiveItem = $this->getMostExpensiveItems($items)) {
                $this->helper->updateAbandonedProductName(
                    $mostExpensiveItem->getName(),
                    $email,
                    $websiteId
                );
            }
            $this->createAbandonedCart($abandonedModel, $quote, $itemIds, self::STATUS_SENT);
            return true;
        }
    }

    /**
     * @param integer $num
     * @param integer $storeId
     *
     * @return boolean
     */
    public function isLostBasketCustomerEnabled($num, $storeId)
    {
        return $this->scopeConfig->isSetFlag(
            constant('self::XML_PATH_LOSTBASKET_CUSTOMER_ENABLED_' . $num),
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $storeId
        );
    }

    /**
     * @param array $allItemsIds
     * @return array
     */
    private function getQuoteItemIds($allItemsIds)
    {
        $itemIds = [];
        foreach ($allItemsIds as $item) {
            $itemIds[] = $item->getProductId();
        }

        return $itemIds;
    }

    /**
     * @param \Pinpoint\Conesso\Model\Abandoned $abandonedModel
     * @param \Magento\Quote\Model\Quote          $quote
     * @param array                               $itemIds
     * @param string                              $status
     */
    private function createAbandonedCart($abandonedModel, $quote, $itemIds, $status)
    {
        $abandonedModel->setStoreId($quote->getStoreId())
            ->setCustomerId($quote->getCustomerId())
            ->setEmail($quote->getCustomerEmail())
            ->setQuoteId($quote->getId())
            ->setQuoteUpdatedAt($quote->getUpdatedAt())
            ->setAbandonedCartNumber(1)
            ->setItemsCount($quote->getItemsCount())
            ->setItemsIds(implode(',', $itemIds))
            ->setStatus($status)
            ->save();
    }

    /**
     * @param \Pinpoint\Conesso\Model\Abandoned $abandonedModel
     *
     * @return mixed
     */
    private function abandonedCartAlreadyExists($abandonedModel)
    {
        return $abandonedModel->getId();
    }

    /**
     * @param \Pinpoint\Conesso\Model\Abandoned $abandonedModel
     * @param \Magento\Quote\Model\Quote          $quote
     * @return boolean
     */
    private function shouldNotSendACAgain($abandonedModel, $quote)
    {
        return
            !$quote->getIsActive() ||
            $quote->getItemsCount() == 0 ||
            !$this->isItemsChanged($quote, $abandonedModel);
    }

    /**
     * @param \Magento\Quote\Model\Quote          $quote
     * @param \Pinpoint\Conesso\Model\Abandoned $abandonedModel
     * @return boolean
     */
    private function isItemsChanged($quote, $abandonedModel)
    {
        if ($quote->getItemsCount() != $abandonedModel->getItemsCount()) {
            return true;
        } else {
            //number of items matches
            $quoteItemIds = $this->getQuoteItemIds($quote->getAllItems());
            $abandonedItemIds = explode(',', $abandonedModel->getItemsIds());

            //quote items not same
            if (! $this->isItemsIdsSame($quoteItemIds, $abandonedItemIds)) {
                return true;
            }

            return false;
        }
    }

    /**
     * Compare items ids.
     *
     * @param array $quoteItemIds
     * @param array $abandonedItemIds
     * @return boolean
     */
    private function isItemsIdsSame($quoteItemIds, $abandonedItemIds)
    {
        return $quoteItemIds == $abandonedItemIds;
    }

    private function isNotAConfirmedContact($abandonedModel)
    {
        return $abandonedModel->getStatus() !== self::STATUS_CONFIRMED;
    }

    /**
     * @param \Magento\Quote\Model\Quote $quote
     * @return boolean
     */
    private function shouldDeleteAbandonedCart($quote)
    {
        return !$quote->getIsActive() || (int) $quote->getItemsCount() === 0;
    }

    /**
     * @param \Dotdigitalgroup\Email\Model\Abandoned $abandonedModel
     * @throws \Exception
     */
    private function deleteAbandonedCart($abandonedModel)
    {
        $this->abandonedResource->delete($abandonedModel);
    }

    /**
     * @param array $items
     * @return boolean|\Magento\Quote\Model\Quote\Item
     */
    public function getMostExpensiveItems($items)
    {
        $mostExpensiveItem = false;
        foreach ($items as $item) {
            /** @var $item \Magento\Quote\Model\Quote\Item */
            if ($mostExpensiveItem == false) {
                $mostExpensiveItem = $item;
            } elseif ($item->getPrice() > $mostExpensiveItem->getPrice()) {
                $mostExpensiveItem = $item;
            }
        }

        return $mostExpensiveItem;
    }

    /**
     * @param $storeId
     * @param $campaignId
     *
     * @return integer
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    private function processConfirmedCustomerAbandonedCart($storeId)
    {
        $ac1QuoteIdsWithConfirmedContacts = $this->abandonedCollectionFactory->create()
            ->getCollectionByConfirmedStatus($storeId)
            ->getColumnValues('quote_id');

        $quoteCollectionFromIds = $this->orderCollection->create()
            ->getStoreQuotesFromQuoteIds($ac1QuoteIdsWithConfirmedContacts, $storeId);
        return $this->createCustomerFirstAbandonedCart(
            $quoteCollectionFromIds,
            $storeId
        );
    }

    /**
     * @param integer $storeId
     * @param integer $num
     *
     * @return \DateInterval
     */
    private function getInterval($storeId, $num)
    {
        if ($num == 1) {
            $minutes = $this->getLostBasketCustomerInterval($num, $storeId);
            $interval = $this->dateIntervalFactory->create(
                ['interval_spec' => sprintf('PT%sM', $minutes)]
            );
        } else {
            $hours = (int)$this->getLostBasketCustomerInterval($num, $storeId);
            $interval = $this->dateIntervalFactory->create(
                ['interval_spec' => sprintf('PT%sH', $hours)]
            );
        }

        return $interval;
    }

    /**
     * @param integer $num
     * @param integer $storeId
     *
     * @return mixed
     */
    public function getLostBasketCustomerInterval($num, $storeId)
    {
        return $this->scopeConfig->getValue(
            constant('self::XML_PATH_LOSTBASKET_CUSTOMER_INTERVAL_' . $num),
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $storeId
        );
    }

    /**
     * @param integer $campaignId
     * @param integer $storeId
     * @param integer $websiteId
     * @param integer $number
     * @param boolean $guest
     *
     * @return integer
     */
    private function processExistingAbandonedCart($storeId, $websiteId, $number, $guest = false)
    {
        $result = 0;
        if ($guest) {
            $interval = $this->getSendAfterIntervalForGuest($storeId, $number);
            $message = 'Guest';
        } else {
            $interval = $this->getInterval($storeId, $number);
            $message = 'Customer';
        }

        $fromTime = $this->getSyncFromTime($interval);
        $toTime = clone $fromTime;
        $fromTime->sub($this->dateIntervalFactory->create(['interval_spec' => 'PT5M']));
        $fromDate   = $fromTime->format('Y-m-d H:i:s');
        $toDate     = $toTime->format('Y-m-d H:i:s');
        //get abandoned carts already sent
        $abandonedCollection = $this->getAbandonedCartsForStore(
            $number,
            $fromDate,
            $toDate,
            $storeId,
            $guest
        );

        //quote collection based on the updated date from abandoned cart table
        $quoteIds = $abandonedCollection->getColumnValues('quote_id');
        if (empty($quoteIds)) {
            return $result;
        }
        $quoteCollection = $this->getProcessedQuoteByIds($quoteIds, $storeId);

        //found abandoned carts
        if ($quoteCollection->getSize()) {
            $this->helper->log(
                $message . ' Abandoned Cart ' . $number . ',from ' . $fromDate . '  :  ' . $toDate . ', storeId '
                . $storeId
            );
        }

        foreach ($quoteCollection as $quote) {

            $this->cartInsight->send($quote, $storeId);

            $quoteId = $quote->getId();
            $email = $quote->getCustomerEmail();

            if ($mostExpensiveItem = $this->getMostExpensiveItems($quote->getAllItems())) {
                $this->helper->updateAbandonedProductName(
                    $mostExpensiveItem->getName(),
                    $email,
                    $websiteId
                );
            }

            $abandonedModel = $this->abandonedFactory->create()
                ->loadByQuoteId($quoteId);
            //number of items changed or not active anymore
            if ($this->isItemsChanged($quote, $abandonedModel)) {
                if ($this->shouldDeleteAbandonedCart($quote)) {
                    $this->deleteAbandonedCart($abandonedModel);
                }
                continue;
            }

            $abandonedModel->setAbandonedCartNumber($number)
                ->setQuoteUpdatedAt($quote->getUpdatedAt())
                ->save();

            $result++;
        }

        return $result;
    }

    /**
     * @param integer $storeId
     * @param integer $num
     *
     * @return \DateInterval
     */
    protected function getSendAfterIntervalForGuest($storeId, $num)
    {
        $timeInterval = $this->getLostBasketSendAfterForGuest($num, $storeId);

        //for the  first cart which use the minutes
        if ($num == 1) {
            $interval = $this->dateIntervalFactory->create(
                ['interval_spec' => sprintf('PT%sM', $timeInterval)]
            );
        } else {
            $interval = $this->dateIntervalFactory->create(
                ['interval_spec' => sprintf('PT%sH', $timeInterval)]
            );
        }

        return $interval;
    }

    /**
     * @param integer $num
     * @param integer $storeId
     *
     * @return mixed
     */
    public function getLostBasketSendAfterForGuest($num, $storeId)
    {
        return $this->scopeConfig->getValue(
            constant('self::XML_PATH_LOSTBASKET_GUEST_INTERVAL_' . $num),
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $storeId
        );
    }

    /**
     * @param integer $number
     * @param string  $from
     * @param string  $to
     * @param integer $storeId
     * @param boolean $guest
     *
     * @return mixed
     */
    private function getAbandonedCartsForStore($number, $from, $to, $storeId, $guest = false)
    {
        $updated = [
            'from' => $from,
            'to'   => $to,
            'date' => true
        ];

        return $this->abandonedCollectionFactory->create()->getAbandonedCartsForStore(
            --$number,
            $storeId,
            $updated,
            self::STATUS_SENT,
            $this->helper->isOnlySubscribersForAC($storeId),
            $guest
        );
    }

    /**
     * @param array   $quoteIds
     * @param integer $storeId
     * @return mixed
     */
    private function getProcessedQuoteByIds($quoteIds, $storeId)
    {
        $quoteCollection = $this->quoteCollectionFactory->create()
            ->addFieldToFilter('entity_id', ['in' => $quoteIds])
            ->addFieldToFilter('is_active', 1);

        //process rules on collection
        $ruleModel       = $this->rulesFactory->create();
        $quoteCollection = $ruleModel->process(
            $quoteCollection,
            \Dotdigitalgroup\Email\Model\Rules::ABANDONED,
            $this->helper->storeManager->getStore($storeId)->getWebsiteId()
        );

        return $quoteCollection;
    }

    protected function syncCarts($website)
    {

        // Initialise a return hash containing results of our sync attempt

        $carts = $this->abandonedCollectionFactory->create()->getCollectionByPendingStatus();
        foreach ($carts->getItems() as $cart) {
            var_dump($cart->getData());
        }

        $message = '(Website '.$website->getId().') Syncing Carts: '.$carts->getSize();

        //create bulk
        if ($carts->getSize()) {
            $syncedInfo = $this->helper->syncBulkCartsWithConesso($carts);
            if ($syncedInfo['success'] == true) {
                //$this->orderResource->setImported($this->orderIds);
                $this->cartsSynced += $carts->getSize();
                $message .= ' Bulk Synced: '.$carts->getSize();
            }

            $this->helper->log('--------- Cart sync ---------- : ' . $carts->getSize());
        }

        $this->messages[] = $message;

        return true;
    }
}
