<?php

namespace Pinpoint\Conesso\Model\Apiconnector;

/**
 * Class ContactImportQueueExport
 * @package Pinpoint\Conesso\Model\Apiconnector
 */
class ContactImportQueueExport
{
    /**
     * @var \Pinpoint\Conesso\Model\ImporterFactory
     */
    public $importerFactory;

    /**
     * @var \Pinpoint\Conesso\Helper\File
     */
    public $file;

    /**
     * ContactImportQueueExport constructor.
     *
     * @param \Pinpoint\Conesso\Model\ImporterFactory $importerFactory
     * @param \Pinpoint\Conesso\Helper\File           $file
     */
    public function __construct(
        \Pinpoint\Conesso\Model\ImporterFactory $importerFactory,
        \Pinpoint\Conesso\Helper\File $file
    ) {
        $this->importerFactory = $importerFactory;
        $this->file            = $file;
    }

    /**
     * @param \Magento\Store\Api\Data\WebsiteInterface        $website
     * @param string                                          $customersFile
     * @param integer                                         $customerNum
     * @param array                                           $customerIds
     * @param \Pinpoint\Conesso\Model\ResourceModel\Contact $resource
     *
     * @return null
     */
    public function enqueueForExport(
        \Magento\Store\Api\Data\WebsiteInterface $website,
        $customersFile,
        $customerNum,
        $customerIds,
        $resource
    ) {
        if (is_file($this->file->getFilePath($customersFile))) {
            if ($customerNum > 0) {
                //register in queue with importer
                $this->importerFactory->create()
                    ->registerQueue(
                        \Pinpoint\Conesso\Model\Importer::IMPORT_TYPE_CONTACT,
                        '',
                        \Pinpoint\Conesso\Model\Importer::MODE_BULK,
                        $website->getId(),
                        $customersFile
                    );

                //set imported
                $resource->setImportedByIds($customerIds);
            }
        }
    }
}
