<?php

namespace Pinpoint\Conesso\Model\Catalog;

class UpdateCatalog
{
    /**
     * @var \Pinpoint\Conesso\Model\ResourceModel\Catalog
     */
    private $catalogResource;

    /**
     * @var \Pinpoint\Conesso\Model\CatalogFactory
     */
    private $catalogFactory;

    /**
     * UpdateCatalog constructor.
     * This class is being using by observers when we add new products manually or via CSV
     * @param \Pinpoint\Conesso\Model\ResourceModel\Catalog $catalogResource
     * @param \Pinpoint\Conesso\Model\CatalogFactory        $catalogFactory
     */
    public function __construct(
        \Pinpoint\Conesso\Model\ResourceModel\Catalog $catalogResource,
        \Pinpoint\Conesso\Model\CatalogFactory $catalogFactory
    ) {
        $this->catalogResource = $catalogResource;
        $this->catalogFactory = $catalogFactory;
    }

    public function execute($productId)
    {
        $emailCatalogModel = $this->catalogFactory->create();
        $emailCatalog = $emailCatalogModel->loadProductById($productId);

        if ($emailCatalog->getId()) {
            $this->updateEmailCatalog($emailCatalog);
        } else {
            $this->createEmailCatalog($emailCatalogModel, $productId);
        }
    }

    /**
     * update email catalog item when imported
     * @param $emailCatalog
     * @throws \Magento\Framework\Exception\AlreadyExistsException
     */
    private function updateEmailCatalog($emailCatalog)
    {
        if ($emailCatalog->getImported()) {
            $emailCatalog->setModified(1);
            $this->catalogResource->save($emailCatalog);
        }
    }

    /**
     * create new email catalog item
     * @param $emailCatalogModel
     * @param $productId
     */
    private function createEmailCatalog($emailCatalogModel, $productId)
    {
        $emailCatalogModel->setProductId($productId);
        $this->catalogResource->save($emailCatalogModel);
    }
}
