<?php

namespace Pinpoint\Conesso\Model;

use Magento\Customer\Model\CustomerFactory;
use Magento\Framework\Stdlib\DateTime;
use Magento\Newsletter\Model\SubscriberFactory;
use Magento\Store\Api\Data\WebsiteInterface;
use Magento\Store\Model\StoreManagerInterface;
use Pinpoint\Conesso\Helper\Data;
use Pinpoint\Conesso\Helper\File;
use Pinpoint\Conesso\Model\AbandonedCart\ProgramEnrolment\Enroller;
use Pinpoint\Conesso\Model\Connector\AccountFactory;
use Pinpoint\Conesso\Model\ContactFactory;
use Pinpoint\Conesso\Model\Email\TemplateFactory;
use Pinpoint\Conesso\Model\ResourceModel\Contact;
use Pinpoint\Conesso\Model\ResourceModel\Contact\CollectionFactory;
use Pinpoint\Conesso\Model\ResourceModel\Order;
use Pinpoint\Conesso\Model\ResourceModel\Order\Collection;

/**
 * @SuppressWarnings(PHPMD.CouplingBetweenObjects)
 */
class Cron
{
    /**
     * @var CollectionFactory
     */
    private $contactCollectionFactory;

    /**
     * @var integer
     */
    private $countCustomers = 0;

    /**
     * @var Email\TemplateFactory
     */
    private $templateFactory;

    /**
     * @var ContactFactory
     */
    private $contactFactory;

    /**
     * @var Sync\AutomationFactory
     */
    private $automationFactory;

    /**
     * @var Sync\ImporterFactory
     */
    private $importerFactory;

    /**
     * @var Sync\CatalogFactory
     */
    private $catalogFactory;

    /**
     * @var SubscriberFactory
     */
    private $subscriberFactory;

    /**
     * @var Customer\GuestFactory
     */
    private $guestFactory;

    /**
     * @var Sales\QuoteFactory
     */
    private $quoteFactory;

    /**
     * @var Sync\OrderFactory
     */
    private $syncOrderFactory;

    /**
     * @var Sync\CampaignFactory
     */
    private $campaignFactory;

    /**
     * @var Data
     */
    private $helper;

    /**
     * @var File
     */
    private $fileHelper;

    /**
     * @var ResourceModel\Importer
     */
    private $importerResource;

    /**
     * @var ResourceModel\Cron\CollectionFactory
     */
    private $cronCollection;

    /**
     * @var Cron\CronSub
     */
    private $cronHelper;

    /**
     * @var mixed
     */
    private $start;

    /**
     * @var integer
     */
    private $importedContacts = 0;

    /**
     * @var CustomerFactory
     */
    protected $customerFactory;

    /**
     * @var DateTime
     */
    private $dateTime;

    /**
     * @var Enroller
     */
    private $abandonedCartProgramEnroller;

    /**
     * @var Contact
     */
    public $contactResource;

    /**
     * @var \Magento\Sales\Model\ResourceModel\Order\CollectionFactory
     */
    protected $orderCollectionFactory;

    /**
     * @var OrderFactory
     */
    private $emailOrderFactory;

    /**
     * @var Order
     */
    private $orderResource;

    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * @var AccountFactory
     */
    private $accountFactory;

    /**
     * @var Account
     */
    private $account;

    /**
     * Global number of orders.
     *
     * @var array
     */
    public $countOrders = [
        'orders' => 0,
        'single_sync' => 0,
        'pending' => 0,
        'modified' => 0,
    ];

    /**
     * @var array
     */
    private $orderIds = [];

    /**
     * @var \Pinpoint\Conesso\Model\Connector\OrderFactory
     */
    private $connectorOrderFactory;

    /**
     * @var array
     */
    public $guests = [];

    /**
     * Cron constructor.
     *
     * @param Sync\CampaignFactory                                       $campaignFactory
     * @param Sync\OrderFactory                                          $syncOrderFactory
     * @param Sales\QuoteFactory                                         $quoteFactory
     * @param Customer\GuestFactory                                      $guestFactory
     * @param SubscriberFactory                                          $subscriberFactory
     * @param Sync\CatalogFactory                                        $catalogFactorty
     * @param Sync\ImporterFactory                                       $importerFactory
     * @param Sync\AutomationFactory                                     $automationFactory
     * @param Apiconnector\ContactFactory                                $contact
     * @param Data                                                       $helper
     * @param File                                                       $fileHelper
     * @param ResourceModel\Importer                                     $importerResource
     * @param Email\TemplateFactory                                      $templateFactory
     * @param ResourceModel\Cron\CollectionFactory                       $cronCollection
     * @param Cron\CronSubFactory                                        $cronSubFactory
     * @param AbandonedCart\ProgramEnrolment\Enroller                    $abandonedCartProgramEnroller
     * @param CollectionFactory                                          $contactCollectionFactory
     * @param \Pinpoint\Conesso\Model\ContactFactory                   $contactFactory
     * @param DateTime                                                   $dateTime
     * @param Contact                                                    $contactResource
     * @param CustomerFactory                                            $customerFactory
     * @param \Magento\Sales\Model\ResourceModel\Order\CollectionFactory $orderCollectionFactory
     * @param OrderFactory                                               $emailOrderFactory
     * @param Order                                                      $orderResource
     * @param StoreManagerInterface                                      $storeManagerInterface
     * @param AccountFactory                                             $accountFactory
     * @param Connector\OrderFactory                                     $connectorOrderFactory
     * @SuppressWarnings(PHPMD.ExcessiveParameterList)
     */
    public function __construct(
        Sync\CampaignFactory $campaignFactory,
        Sync\OrderFactory $syncOrderFactory,
        Sales\QuoteFactory $quoteFactory,
        Customer\GuestFactory $guestFactory,
        SubscriberFactory $subscriberFactory,
        Sync\CatalogFactory $catalogFactorty,
        Sync\ImporterFactory $importerFactory,
        Sync\AutomationFactory $automationFactory,
        Apiconnector\ContactFactory $contact,
        Data $helper,
        File $fileHelper,
        ResourceModel\Importer $importerResource,
        TemplateFactory $templateFactory,
        ResourceModel\Cron\CollectionFactory $cronCollection,
        Cron\CronSubFactory $cronSubFactory,
        Enroller $abandonedCartProgramEnroller,
        CollectionFactory $contactCollectionFactory,
        ContactFactory $contactFactory,
        DateTime $dateTime,
        Contact $contactResource,
        CustomerFactory $customerFactory,
        \Magento\Sales\Model\ResourceModel\Order\CollectionFactory $orderCollectionFactory,
        OrderFactory $emailOrderFactory,
        Order $orderResource,
        StoreManagerInterface $storeManagerInterface,
        AccountFactory $accountFactory,
        Connector\OrderFactory $connectorOrderFactory
    ) {
        $this->campaignFactory   = $campaignFactory;
        $this->syncOrderFactory  = $syncOrderFactory;
        $this->quoteFactory      = $quoteFactory;
        $this->guestFactory      = $guestFactory;
        $this->subscriberFactory = $subscriberFactory;
        $this->catalogFactory    = $catalogFactorty;
        $this->importerFactory   = $importerFactory;
        $this->automationFactory = $automationFactory;
        $this->contactFactory    = $contact;
        $this->helper            = $helper;
        $this->fileHelper        = $fileHelper;
        $this->importerResource  = $importerResource;
        $this->cronCollection    = $cronCollection;
        $this->templateFactory   = $templateFactory;
        $this->cronHelper        = $cronSubFactory->create();
        $this->abandonedCartProgramEnroller = $abandonedCartProgramEnroller;
        $this->contactCollectionFactory = $contactCollectionFactory;
        $this->contactFactory   = $contactFactory;
        $this->dateTime = $dateTime;
        $this->contactResource = $contactResource;
        $this->customerFactory = $customerFactory;
        $this->orderCollectionFactory = $orderCollectionFactory;
        $this->emailOrderFactory      = $emailOrderFactory;
        $this->orderResource = $orderResource;
        $this->storeManager = $storeManagerInterface;
        $this->accountFactory = $accountFactory;
        $this->account = $this->accountFactory->create();
        $this->connectorOrderFactory = $connectorOrderFactory;
    }

    /**
     * @return array
     */
    public function importContacts()
    {

        $result = ['success' => true, 'message' => 'success cron'];

        try {
            $collection = $this->customerFactory->create()->getCollection()
                ->addAttributeToSelect("*")
                ->addFieldToFilter(
                    'entity_id',
                    ['nin' => $this->importedContacts()]
                )
                ->load();
            foreach ($collection as $customer) {
                $customerId = $customer->getId();
                $websiteId  = $customer->getWebsiteId();
                $storeId = $customer->getStoreId();
                $email = $customer->getEmail();
                $subscriber = $this->subscriberFactory->create()
                    ->loadByCustomerId($customerId);
                $isSubscribed = $subscriber->isSubscribed();

                //check if enabled
                if (!$this->helper->isEnabled($websiteId)) {
                    return ['success' => false, 'message' => 'Api not enabled'];
                }

                $emailBefore = '';

                $isContactExist = $this->contactFactory->create()->loadByCustomerId($customerId);
                if ($isContactExist->getIimportContactsd()) {
                    $emailBefore = $isContactExist->getEmail();
                }

                if ($emailBefore !== $email) {
                    $contactModel = $this->contactFactory->create()
                        ->loadByCustomerEmail($email, $websiteId);
                    $contactModel
                        ->setEmail($email)
                        ->setEmailImported(\Pinpoint\Conesso\Model\Contact::EMAIL_CONTACT_NOT_IMPORTED)
                        ->setStoreId($storeId)
                        ->setCustomerId($customerId);
                    if ($isSubscribed) {
                        $contactModel->setLastSubscribedAt($this->dateTime->formatDate(true))
                            ->setIsSubscriber(1);
                    }
                    $this->contactResource->save($contactModel);
                    $this->importedContacts++;
                }
            }

        } catch (\Exception $e) {
            $this->helper->log($e->getMessage());
        }

        return $result;
    }

    /**
     * @return array
     */
    protected function importedContacts()
    {
        $importedIDs = [0];
        $collection = $this->contactFactory->create()->getCollection();
        if ($collection->getSize() > 0) {
            foreach ($collection->getItems() as $item) {
                $importedIDs[] = $item->getCustomerId();
            }
        }
        return $importedIDs;
    }

    /**
     * CRON FOR CONTACTS SYNC.
     *
     * @return array
     */
    public function contactSync()
    {
        $result = ['success' => true, 'message' => 'success cron'];
        try {
            //starting time for sync
            $this->start = microtime(true);
            //export bulk contacts
            foreach ($this->helper->getWebsites() as $website) {
                $apiEnabled = $this->helper->isEnabled($website);
                $customerSyncEnabled = $this->helper->isCustomerSyncEnabled(
                    $website
                );
                $subscriberSyncEnabled = $this->helper->isSubscriberSyncEnabled(
                    $website
                );
                if ($apiEnabled && ($customerSyncEnabled || $subscriberSyncEnabled)) {
                    //start log
                    $contactsUpdated = $this->syncContacts($website);

                    // show message for any number of customers
                    if ($contactsUpdated) {
                        $result['message'] .=  $website->getName()
                            . ', updated contacts ' . $contactsUpdated;
                    }
                }
            }

            //sync proccessed
            if ($this->countCustomers) {
                $message = '----------- Customer sync ----------- : ' .
                    gmdate('H:i:s', microtime(true) - $this->start) .
                    ', Total contacts = ' . $this->countCustomers;
                $this->helper->log($message);
                $message .= $result['message'];
                $result['message'] = $message;
            }
        } catch (\Exception $e) {
            $this->helper->log($e->getMessage());
        }

        return $result;
    }

    /**
     * @param WebsiteInterface $website
     * @return boolean|integer
     */
    protected function syncContacts(WebsiteInterface $website)
    {

        $syncLimit = $this->helper->getSyncLimit($website);

        $onlySubscribers = $this->helper->isOnlySubscribersForContactSync($website->getId());
        $contacts = $this->contactCollectionFactory->create();
        $contacts = ($onlySubscribers) ? $contacts->getContactsToImportByWebsite($website->getId(), $syncLimit, true) :
            $contacts->getContactsToImportByWebsite($website->getId(), $syncLimit);

        // no contacts found
        if (!$contacts->getSize()) {
            return 0;
        }

        // Sync Contacts
        foreach ($contacts->getItems() as $contact) {
            if ($contact->getData()) {
                $syncedInfo = $this->helper->syncContactWithConesso($contact);
                if ($syncedInfo['success'] === true) {
                    $contactInfo = $contact->getData();
                    $contact = $this->contactFactory->create();
                    $contact->load($contactInfo['conesso_contact_id']);
                    $contact->setConessoImported(1);
                    $contact->setSubscriberImported(1);
                    $contact->setContactId($syncedInfo['contact_id']);
                    $contact->save();
                    $this->countCustomers++;
                }
            }
        }
        return true;
    }

    /**
     * @return $this
     * @throws NoSuchEntityException
     * @throws AlreadyExistsException
     */
    public function importOrders()
    {
        $collection = $this->orderCollectionFactory->create()
            ->addFieldToFilter(
                'entity_id',
                ['nin' => $this->importedOrders()]
            );
        foreach ($collection as $order) {
            $status         = $order->getStatus();
            $storeId        = $order->getStoreId();
            $store      = $this->storeManager->getStore($storeId);
            $emailOrder = $this->emailOrderFactory->create()
                ->loadByOrderId($order->getEntityId(), $order->getQuoteId());
            //reimport email order
            $emailOrder->setUpdatedAt($order->getUpdatedAt())
                ->setCreatedAt($order->getUpdatedAt())
                ->setStoreId($store->getId())
                ->setOrderStatus($status);

            if ($emailOrder->getConessoImported() !== \Pinpoint\Conesso\Model\Contact::EMAIL_CONTACT_IMPORTED) {
                $emailOrder->setConessoImported(null);
            }
            $isEnabled = $this->helper->isStoreEnabled($store->getId());
            //api not enabled, stop emulation and exit
            if (!$isEnabled) {
                return $this;
            }
            // set back the current store
            $this->orderResource->save($emailOrder);
        }
        return $this;
    }

    /**
     * @return array
     */
    protected function importedOrders()
    {
        $importedIDs = [0];
        $collection = $this->emailOrderFactory->create()->getCollection();
        if ($collection->getSize() > 0) {
            foreach ($collection->getItems() as $item) {
                $importedIDs[] = $item->getOrderId();
            }
        }
        return $importedIDs;
    }

    /**
     * CRON FOR CATALOG SYNC.
     * @return array
     */
    public function catalogSync()
    {
        if ($this->jobHasAlreadyBeenRun('conesso_automation_catalog_sync')) {
            $message = 'Skipping conesso_automation_catalog_sync job run';
            $this->helper->log($message);
            return ['message' => $message];
        }
        $result = $this->catalogFactory->create()
            ->sync();
        return $result;
    }

    /**
     * CRON FOR ABANDONED CARTS.
     * @return void
     */
    public function abandonedCarts()
    {
        if ($this->jobHasAlreadyBeenRun('conesso_automation_abandonedcarts')) {
            $this->helper->log('Skipping conesso_automation_abandonedcarts job run');
            return;
        }
        $this->quoteFactory->create()->processAbandonedCarts();
        $this->abandonedCartProgramEnroller->process();
    }

    /**
     * CRON FOR ORDER TRANSACTIONAL DATA.
     *
     * @return array
     */
    public function orderSync()
    {

        $result = ['success' => true, 'message' => 'success cron'];
        try {
            //starting time for sync
            $this->start = microtime(true);
            //export bulk contacts
            foreach ($this->helper->getWebsites() as $website) {
                $apiEnabled = $this->helper->isEnabled($website);
                $orderSyncEnabled = $this->helper->isOrderSyncEnabled(
                    $website
                );
                if ($apiEnabled && $orderSyncEnabled) {
                    //start log
                    $ordersUpdated = $this->syncOrders($website);

                    // show message for any number of customers
                    if ($ordersUpdated) {
                        $result['message'] .=  $website->getName()
                            . ', updated orders ' . $ordersUpdated;
                    }
                }
            }

        } catch (\Exception $e) {
            $this->helper->log($e->getMessage());
        }

        return $result;
    }

    /**
     * @param WebsiteInterface $website
     * @return boolean
     * @throws NoSuchEntityException
     */
    protected function syncOrders(WebsiteInterface $website)
    {

        // Initialise a return hash containing results of our sync attempt
        $this->searchWebsiteAccounts($website);

        $orders = $this->account->getOrders();

        $ordersForSingleSync = $this->account->getOrdersForSingleSync();
        $numOrders = count($orders);
        $numOrdersForSingleSync = count($ordersForSingleSync);
        $totalOrders = $numOrders + $numOrdersForSingleSync;
        $message = '(Website '.$website->getId().') Syncing Orders: '.$totalOrders;
        $this->helper->log($message);
        $this->countOrders['orders'] += $numOrders;
        $this->countOrders['single_sync'] += $numOrdersForSingleSync;

        //create bulk
        if ($numOrders) {
            $syncedInfo = $this->helper->syncBulkOrdersWithConesso($orders);
            if ($syncedInfo['success'] === true) {
                $this->orderResource->setImported($this->orderIds);
            }

            $this->helper->log('--------- Order sync ---------- : ' . $numOrders);
        }
        //create single
        if ($numOrdersForSingleSync) {
            $this->helper->log('SYNC SINGLE ORDERS');
            $syncedInfo = $this->helper->syncBulkOrdersWithConesso($orders);
            if ($syncedInfo['success'] === true) {
                $this->orderResource->setImported($this->orderIds);
            }
        }

        //mark the orders as imported
        //$this->orderResource->setImported($this->orderIds);
        //unset($this->account);
        /**
         * Add guests to contact table.
         */
        if (!empty($this->guests)) {
            $orderEmails = array_keys($this->guests);
            $guestsEmailFound = $this->contactCollectionFactory->create()
                ->addFieldToFilter('email', ['in' => $orderEmails])
                ->getColumnValues('email');
            //remove the contacts that already exists
            foreach ($guestsEmailFound as $email) {
                unset($this->guests[strtolower($email)]);
            }

            //insert new guests contacts
            $this->contactResource->insertGuests($this->guests);
            //mark the existing contacts with is guest in bulk
            $this->contactResource->updateContactsAsGuests($guestsEmailFound);
        }
        unset($this->account);
        return true;
    }

    /**
     * @param WebsiteInterface $website
     * @throws NoSuchEntityException
     * @return void
     */
    public function searchWebsiteAccounts(WebsiteInterface $website)
    {
        $this->account = $this->accountFactory->create();
        $this->apiUsername = $this->helper->getApiUsername($website);
        $this->apiPassword = $this->helper->getApiPassword($website);
        // limit for orders included to sync
        $limit = $this->helper->getWebsiteConfig(
            \Pinpoint\Conesso\Helper\Config::XML_PATH_CONNECTOR_TRANSACTIONAL_DATA_SYNC_LIMIT,
            $website
        );

        $pendingOrders = $this->getPendingConnectorOrders($website, $limit);
        if (!empty($pendingOrders)) {
            $this->countOrders['pending'] += count($pendingOrders);
            $this->account->setOrders($pendingOrders);
        }
        $this->account->setWebsites($website->getId());

        $modifiedOrders = $this->getModifiedOrders($website, $limit);
        if (! empty($modifiedOrders)) {
            $this->countOrders['modified'] += count($modifiedOrders);
            $this->account->setOrdersForSingleSync($modifiedOrders);
        }
    }

    /**
     * @param WebsiteInterface $website
     * @param integer          $limit
     * @return array
     * @throws NoSuchEntityException
     */
    public function getPendingConnectorOrders(WebsiteInterface $website, int $limit = 100)
    {
        $orders = [];
        $storeIds = $website->getStoreIds();
        $orderModel = $this->emailOrderFactory->create();
        //get order statuses set in configuration section
        $orderStatuses = $this->helper->getConfigSelectedStatus($website);

        //no active store for website
        if (empty($storeIds) || empty($orderStatuses)) {
            return [];
        }

        //pending order from conesso_order
        $orderCollection = $orderModel->getOrdersToImport($storeIds, $limit, $orderStatuses);

        //no orders found
        if (! $orderCollection->getSize()) {
            return $orders;
        }

        $orders = $this->mapOrderData($orderCollection, $orderModel, $orders);

        return $orders;
    }

    /**
     * @param Collection                      $orderCollection
     * @param \Pinpoint\Conesso\Model\Order $orderModel
     * @param array                           $orders
     * @return array
     * @throws NoSuchEntityException
     */
    protected function mapOrderData(
        Collection $orderCollection,
        \Pinpoint\Conesso\Model\Order $orderModel,
        array $orders
    ) {
        $orderIds = $orderCollection->getColumnValues('order_id');

        //get the order collection
        $salesOrderCollection = $orderModel->getSalesOrdersWithIds($orderIds);

        foreach ($salesOrderCollection as $order) {
            if ($order->getId()) {
                $storeId = $order->getStoreId();
                $websiteId = $this->helper->storeManager->getStore($storeId)->getWebsiteId();

                /**
                 * Add guest to array to add to contacts table.
                 */
                if ($order->getCustomerIsGuest() && $order->getCustomerEmail()) {
                    $email = $order->getCustomerEmail();
                    $this->guests[strtolower($email)] = [
                        'email' => $email,
                        'website_id' => $websiteId,
                        'store_id' => $storeId,
                        'is_guest' => 1
                    ];
                }

                $connectorOrder = $this->connectorOrderFactory->create();
                $connectorOrder->setOrderData($order);
                $orders[] = $connectorOrder;
            }

            $this->orderIds[] = $order->getId();
        }

        return $orders;
    }

    /**
     * @param WebsiteInterface $website
     * @param integer          $limit
     *
     * @return array
     */
    protected function getModifiedOrders(WebsiteInterface $website, int $limit)
    {
        $orders =  [];
        $storeIds = $website->getStoreIds();
        /** @var \Pinpoint\Conesso\Model\Order $orderModel */
        $orderModel = $this->emailOrderFactory->create();
        //get order statuses set in configuration section
        $orderStatuses = $this->helper->getConfigSelectedStatus($website);

        //no active store for website
        if (empty($storeIds) || empty($orderStatuses)) {
            return [];
        }

        //pending order from email_order
        $orderCollection = $orderModel->getModifiedOrdersToImport($storeIds, $limit, $orderStatuses);

        //no orders found
        if (! $orderCollection->getSize()) {
            return $orders;
        }

        $orders = $this->mapOrderData($orderCollection, $orderModel, $orders);

        return $orders;
    }

    /**
     * Check if already ran for same time
     *
     * @param string $jobCode
     * @return boolean
     */
    private function jobHasAlreadyBeenRun(string $jobCode)
    {
        $currentRunningJob = $this->cronCollection->create()
            ->addFieldToFilter('job_code', $jobCode)
            ->addFieldToFilter('status', 'running')
            ->setPageSize(1);

        if ($currentRunningJob->getSize()) {
            $jobOfSameTypeAndScheduledAtDateAlreadyExecuted =  $this->cronCollection->create()
                ->addFieldToFilter('job_code', $jobCode)
                ->addFieldToFilter('scheduled_at', $currentRunningJob->getFirstItem()->getScheduledAt())
                ->addFieldToFilter('status', ['in' => ['success', 'failed']]);

            return ($jobOfSameTypeAndScheduledAtDateAlreadyExecuted->getSize()) ? true : false;
        }

        return false;
    }
}
