<?php

namespace Pinpoint\Conesso\Model\ResourceModel\Catalog;

class Collection extends \Magento\Framework\Model\ResourceModel\Db\Collection\AbstractCollection
{
    /**
     * @var string
     */
    protected $_idFieldName = 'id';

    /**
     * @var \Pinpoint\Conesso\Helper\Data
     */
    protected $helper;

    /**
     * @var \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory
     */
    public $productCollection;

    /**
     * Initialize resource collection.
     *
     * @return null
     */
    public function _construct()
    {
        $this->_init(
            \Pinpoint\Conesso\Model\Catalog::class,
            \Pinpoint\Conesso\Model\ResourceModel\Catalog::class
        );
    }

    /**
     * Collection constructor.
     *
     * @param \Magento\Framework\Data\Collection\EntityFactoryInterface      $entityFactory
     * @param \Psr\Log\LoggerInterface                                       $logger
     * @param \Magento\Framework\Data\Collection\Db\FetchStrategyInterface   $fetchStrategy
     * @param \Magento\Framework\Event\ManagerInterface                      $eventManager
     * @param \Pinpoint\Conesso\Helper\Data                                $helper
     * @param \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory $productCollection
     * @param \Magento\Framework\DB\Adapter\AdapterInterface|null            $connection
     * @param \Magento\Framework\Model\ResourceModel\Db\AbstractDb|null      $resource
     */
    public function __construct(
        \Magento\Framework\Data\Collection\EntityFactoryInterface $entityFactory,
        \Psr\Log\LoggerInterface $logger,
        \Magento\Framework\Data\Collection\Db\FetchStrategyInterface $fetchStrategy,
        \Magento\Framework\Event\ManagerInterface $eventManager,
        \Pinpoint\Conesso\Helper\Data $helper,
        \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory $productCollection,
        \Magento\Framework\DB\Adapter\AdapterInterface $connection = null,
        \Magento\Framework\Model\ResourceModel\Db\AbstractDb $resource = null
    ) {
        $this->helper = $helper;
        $this->productCollection        = $productCollection;
        parent::__construct(
            $entityFactory,
            $logger,
            $fetchStrategy,
            $eventManager,
            $connection,
            $resource
        );
    }

    /**
     * Get product collection to export.
     *
     * @param null|string|boolean|integer|\Magento\Store\Model\Store $store
     * @param integer                                                $limit
     *
     * @return \Magento\Catalog\Model\ResourceModel\Product\Collection|array
     */
    public function getProductsToExportByStore($store, $limit)
    {
        $connectorCollection = $this;
        $connectorCollection->addFieldToFilter(
            ['imported', 'modified'],
            [
                ['null' => 'true'],
                ['eq' => '1']
            ]
        );
        $connectorCollection->getSelect()->limit($limit);
        $connectorCollection->setOrder(
            'product_id',
            'asc'
        );

        //check number of products
        if ($connectorCollection->getSize()) {
            $productIds = $connectorCollection->getColumnValues('product_id');

            $productCollection = $this->productCollection->create()
                ->addAttributeToSelect('*')
                ->addAttributeToFilter(
                    'entity_id',
                    ['in' => $productIds]
                )->addUrlRewrite();

            if (!empty($store)) {
                $productCollection->addStoreFilter($store);
            }

            //visibility filter
            if ($visibility = $this->helper->getWebsiteConfig(
                \Pinpoint\Conesso\Helper\Config::XML_PATH_CONNECTOR_SYNC_CATALOG_VISIBILITY
            )
            ) {
                $visibility = explode(',', $visibility);
                //remove the default option from values
                $visibility = array_filter($visibility);
                $productCollection->addAttributeToFilter(
                    'visibility',
                    ['in' => $visibility]
                );
            }
            //type filter
            if ($type = $this->helper->getWebsiteConfig(
                \Pinpoint\Conesso\Helper\Config::XML_PATH_CONNECTOR_SYNC_CATALOG_TYPE
            )
            ) {
                $type = explode(',', $type);
                $productCollection->addAttributeToFilter(
                    'type_id',
                    ['in' => $type]
                );
            }

            $productCollection->addWebsiteNamesToResult()
                ->addCategoryIds()
                ->addOptionsToResult();

            $productCollection->clear();

            return $productCollection;
        }

        return [];
    }
}
