<?php

namespace Pinpoint\Conesso\Model;

use Magento\Store\Model\Website;

class Subscriber extends \Magento\Framework\Model\AbstractModel
{
    const EMAIL_CONTACT_IMPORTED = 1;
    const EMAIL_CONTACT_NOT_IMPORTED = null;
    const EMAIL_SUBSCRIBER_NOT_IMPORTED = null;

    /**
     * Constructor.
     *
     * @return void
     */
    public function _construct()
    {
        $this->_init(\Pinpoint\Conesso\Model\ResourceModel\Subscriber::class);
    }

    /**
     * Load contact by customer id.
     *
     * @param integer $customerId
     *
     * @return $this
     */
    public function loadByCustomerId(int $customerId)
    {
        $contact = $this->getCollection()
            ->loadByCustomerId($customerId);

        if ($contact) {
            return $contact;
        }

        return $this;
    }

    /**
     * Get all customer contacts not imported for a website.
     *
     * @param integer $websiteId
     * @param integer $pageSize
     *
     * @return \Pinpoint\Conesso\Model\ResourceModel\Contact\Collection
     */
    public function getContactsToImportForWebsite(int $websiteId, int $pageSize = 100)
    {
        return $this->getCollection()
            ->getContactsToImportForWebsite($websiteId, $pageSize);
    }

    /**
     * Get missing contacts.
     *
     * @param integer $websiteId
     * @param integer $pageSize
     *
     * @return \Pinpoint\Conesso\Model\ResourceModel\Contact\Collection
     */
    public function getMissingContacts(int $websiteId, int $pageSize = 100)
    {
        return $this->getCollection()
            ->getMissingContacts($websiteId, $pageSize);
    }

    /**
     * Load Contact by Email.
     *
     * @param string  $email
     * @param integer $websiteId
     *
     * @return $this
     */
    public function loadByCustomerEmail(string $email, int $websiteId)
    {
        $customer = $this->getCollection()
            ->loadByCustomerEmail($email, $websiteId);

        if ($customer) {
            return $customer;
        } else {
            return $this->setEmail($email)
                ->setWebsiteId($websiteId);
        }
    }

    /**
     * Contact subscribers to import for store.
     *
     * @param integer $storeId
     * @param integer $limit
     * @param boolean $isCustomerCheck
     *
     * @return \Pinpoint\Conesso\Model\ResourceModel\Contact\Collection
     */
    public function getSubscribersToImport(
        int $storeId,
        int $limit = 1000,
        bool $isCustomerCheck = true
    ) {
        return $this->getCollection()
            ->getSubscribersToImport(
                $storeId,
                $limit,
                $isCustomerCheck
            );
    }

    /**
     * Contact subscribers to import for website.
     *
     * @param array $emails
     *
     * @return \Pinpoint\Conesso\Model\ResourceModel\Contact\Collection
     */
    public function getSubscribersToImportFromEmails(array $emails)
    {
        return $this->getCollection()
            ->getSubscribersToImportFromEmails($emails);
    }

    /**
     * Get all not imported guests for a website.
     *
     * @param Website $website
     * @param boolean $onlySubscriber
     *
     * @return \Pinpoint\Conesso\Model\ResourceModel\Contact\Collection
     */
    public function getGuests(Website $website, bool $onlySubscriber = false)
    {
        return $this->getCollection()
            ->getGuests($website->getId(), $onlySubscriber);
    }

    /**
     * Number contacts marked as imported.
     *
     * @return integer
     */
    public function getNumberOfImportedContacs()
    {
        return $this->getCollection()
            ->getNumberOfImportedContacts();
    }

    /**
     * Get the number of customers for a website.
     *
     * @param integer $websiteId
     *
     * @return integer
     */
    public function getNumberCustomerContacts(int $websiteId = 0)
    {
        return $this->getCollection()
            ->getNumberCustomerContacts($websiteId);
    }

    /**
     * Get number of suppressed contacts as customer.
     *
     * @param integer $websiteId
     *
     * @return integer
     */
    public function getNumberCustomerSuppressed(int $websiteId = 0)
    {
        return $this->getCollection()
            ->getNumberCustomerSuppressed($websiteId);
    }

    /**
     * Get number of synced customers.
     *
     * @param integer $websiteId
     *
     * @return integer
     */
    public function getNumberCustomerSynced(int $websiteId = 0)
    {
        return $this->getCollection()
            ->getNumberCustomerSynced($websiteId);
    }

    /**
     * Get number of subscribers synced.
     *
     * @param integer $websiteId
     *
     * @return integer
     */
    public function getNumberSubscribersSynced(int $websiteId = 0)
    {
        return $this->getCollection()
            ->getNumberSubscribersSynced($websiteId);
    }

    /**
     * Get number of subscribers.
     *
     * @param integer $websiteId
     *
     * @return integer
     */
    public function getNumberSubscribers(int $websiteId = 0)
    {
        return $this->getCollection()
            ->getNumberSubscribers($websiteId);
    }
}
