<?php

namespace Pinpoint\Conesso\Model\Sync\Catalog;

class StoreCatalogSyncer
{
    /**
     * @var \Pinpoint\Conesso\Model\ImporterFactory
     */
    private $importerFactory;

    /**
     * @var \Pinpoint\Conesso\Helper\Data
     */
    private $helper;

    /**
     * @var Exporter
     */
    private $exporter;

    /**
     * StoreLevelCatalogSyncer constructor.
     *
     * @param \Pinpoint\Conesso\Model\ImporterFactory $importerFactory
     * @param \Pinpoint\Conesso\Helper\Data           $helper
     * @param Exporter                                  $exporter
     */
    public function __construct(
        \Pinpoint\Conesso\Model\ImporterFactory $importerFactory,
        \Pinpoint\Conesso\Helper\Data $helper,
        Exporter $exporter
    ) {
        $this->importerFactory = $importerFactory;
        $this->helper = $helper;
        $this->exporter = $exporter;
    }

    /**
     * Sync by store
     *
     * @param integer|null $storeId
     * @param integer      $websiteId
     * @param integer      $limit
     * @param string       $importType
     *
     * @return array
     */
    public function syncByStore($storeId, $websiteId, $limit, $importType)
    {
        $products = $this->exporter->exportCatalog($storeId, $limit);

        $success = $this->importerFactory->create()
            ->registerQueue(
                $importType,
                $products,
                \Pinpoint\Conesso\Model\Importer::MODE_BULK,
                $websiteId
            );

        if ($success) {
            return $products;
        } else {
            $pid = implode(",", array_keys($products));
            $msg = "Failed to register with IMPORTER. Type(Catalog) / Scope(Bulk) / Store($storeId) / Product Ids($pid)";
            $this->helper->log($msg);
        }

        return [];
    }
}
