<?php

namespace Pinpoint\Conesso\Observer\Customer;

/**
 * Creates and updates the contact for customer. Monitor the email change for customer.
 * @SuppressWarnings(PHPMD.CouplingBetweenObjects)
 */
class CreateUpdateContact implements \Magento\Framework\Event\ObserverInterface
{

    /**
     * @var \Pinpoint\Conesso\Model\ResourceModel\Catalog
     */
    private $contactResource;

    /**
     * @var \Pinpoint\Conesso\Helper\Data
     */
    private $helper;

    /**
     * @var \Magento\Framework\Registry
     */
    private $registry;

    /**
     * @var \Magento\Customer\Model\CustomerFactory
     */
    private $customerFactory;

    /**
     * @var \Pinpoint\Conesso\Model\ContactFactory
     */
    private $contactFactory;

    /**
     * @var \Magento\Wishlist\Model\WishlistFactory
     */
    private $wishlist;

    /**
     * @var \Pinpoint\Conesso\Model\ImporterFactory
     */
    private $importerFactory;

    /**
     * @var \Magento\Newsletter\Model\SubscriberFactory
     */
    private $subscriberFactory;

    /**
     * @var \Magento\Framework\Stdlib\DateTime
     */
    private $dateTime;

    /**
     * CreateUpdateContact constructor.
     *
     * @param \Magento\Wishlist\Model\WishlistFactory         $wishlist
     * @param \Pinpoint\Conesso\Model\ContactFactory        $contactFactory
     * @param \Magento\Customer\Model\CustomerFactory         $customerFactory
     * @param \Magento\Framework\Registry                     $registry
     * @param \Pinpoint\Conesso\Helper\Data                 $data
     * @param \Pinpoint\Conesso\Model\ResourceModel\Contact $contactResource
     * @param \Pinpoint\Conesso\Model\ImporterFactory       $importerFactory
     * @param \Magento\Newsletter\Model\SubscriberFactory     $subscriberFactory
     */
    public function __construct(
        \Magento\Wishlist\Model\WishlistFactory $wishlist,
        \Pinpoint\Conesso\Model\ContactFactory $contactFactory,
        \Magento\Customer\Model\CustomerFactory $customerFactory,
        \Magento\Framework\Registry $registry,
        \Pinpoint\Conesso\Helper\Data $data,
        \Pinpoint\Conesso\Model\ResourceModel\Contact $contactResource,
        \Pinpoint\Conesso\Model\ImporterFactory $importerFactory,
        \Magento\Newsletter\Model\SubscriberFactory $subscriberFactory,
        \Magento\Framework\Stdlib\DateTime $dateTime
    ) {
        $this->wishlist        = $wishlist;
        $this->contactFactory  = $contactFactory;
        $this->contactResource = $contactResource;
        $this->customerFactory = $customerFactory;
        $this->helper          = $data;
        $this->registry        = $registry;
        $this->importerFactory = $importerFactory;
        $this->subscriberFactory = $subscriberFactory;
        $this->dateTime = $dateTime;
    }

    /**
     * If it's configured to capture on shipment - do this.
     *
     * @param \Magento\Framework\Event\Observer $observer
     *
     * @return $this
     */
    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        $customer = $observer->getEvent()->getCustomer();
        $websiteId  = $customer->getWebsiteId();
        $storeId = $customer->getStoreId();

        //check if enabled
        if (!$this->helper->isEnabled($websiteId)) {
            return $this;
        }

        $email      = $customer->getEmail();
        $customerId = $customer->getEntityId();
        $subscriber = $this->subscriberFactory->create()
            ->loadByCustomerId($customerId);
        $isSubscribed = $subscriber->isSubscribed();

        $emailBefore = '';

        try {
            // fix for a multiple hit of the observer
            $emailReg = $this->registry->registry($email . '_conesso_customer_save');

            if ($emailReg) {
                return $this;
            }
            $this->registry->unregister($email . '_conesso_customer_save'); // additional measure
            $this->registry->register($email . '_conesso_customer_save', $email);

            $isContactExist = $this->contactFactory->create()
                ->loadByCustomerId($customerId);

            if ($isContactExist->getId()) {
                $emailBefore = $isContactExist->getEmail();
            }

            $emailAddress = empty($emailBefore) ? $email : $emailBefore;

            $contactModel = $this->contactFactory->create()
                ->loadByCustomerEmail($emailAddress, $websiteId);

            //email change detection
            if ($emailBefore && $email != $emailBefore) {
                // update email
                $contactModel->setEmail($email);
            } elseif (!$emailBefore) {
                //for new contacts update email
                $contactModel->setEmail($email);
            }

            $contactModel->setConessoImported(\Pinpoint\Conesso\Model\Contact::EMAIL_CONTACT_NOT_IMPORTED)
                ->setStoreId($storeId)
                ->setCustomerId($customerId);

            if ($isSubscribed) {
                $contactModel->setLastSubscribedAt($this->dateTime->formatDate(true));
            }

            $this->contactResource->save($contactModel);
        } catch (\Exception $e) {
            $this->helper->debug((string)$e, []);
        }

        return $this;
    }
}
