<?php

namespace Pinpoint\Conesso\Observer\Customer;

/**
 * Removes the contact if the customer is deleted.
 */
class RemoveContact implements \Magento\Framework\Event\ObserverInterface
{
    /**
     * @var \Pinpoint\Conesso\Helper\Data
     */
    private $helper;

    /**
     * @var \Pinpoint\Conesso\Model\ContactFactory
     */
    private $contactFactory;

    /**
     * @var \Pinpoint\Conesso\Model\ImporterFactory
     */
    private $importerFactory;

    /**
     * @var \Pinpoint\Conesso\Model\ResourceModel\Contact
     */
    private $contactResource;

    /**
     * RemoveContact constructor.
     *
     * @param \Pinpoint\Conesso\Model\ResourceModel\Contact $contactResource
     * @param \Pinpoint\Conesso\Model\ImporterFactory       $importerFactory
     * @param \Pinpoint\Conesso\Model\ContactFactory        $contactFactory
     * @param \Pinpoint\Conesso\Helper\Data                 $data
     */
    public function __construct(
        \Pinpoint\Conesso\Model\ResourceModel\Contact $contactResource,
        \Pinpoint\Conesso\Model\ImporterFactory $importerFactory,
        \Pinpoint\Conesso\Model\ContactFactory $contactFactory,
        \Pinpoint\Conesso\Helper\Data $data
    ) {
        $this->contactFactory  = $contactFactory;
        $this->importerFactory = $importerFactory;
        $this->helper          = $data;
        $this->contactResource = $contactResource;
    }

    /**
     * If it's configured to capture on shipment - do this.
     *
     * @param \Magento\Framework\Event\Observer $observer
     *
     * @return $this
     */
    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        $customer = $observer->getEvent()->getCustomer();
        $email = $customer->getEmail();
        $websiteId = $customer->getWebsiteId();
        $apiEnabled = $this->helper->isEnabled($websiteId);
        $customerSync = $this->helper->isCustomerSyncEnabled($websiteId);

        /*
         * Remove contact.
         */
        if ($apiEnabled && $customerSync) {
            try {
                //register in queue with importer
                $this->importerFactory->create()->registerQueue(
                    \Pinpoint\Conesso\Model\Importer::IMPORT_TYPE_CONTACT,
                    $email,
                    \Pinpoint\Conesso\Model\Importer::MODE_CONTACT_DELETE,
                    $websiteId
                );
                $contactModel = $this->contactFactory->create()
                    ->loadByCustomerEmail($email, $websiteId);
                if ($contactModel->getId()) {
                    //remove contact
                    $this->contactResource->delete($contactModel);
                }
            } catch (\Exception $e) {
                $this->helper->debug((string)$e, []);
            }
        }

        return $this;
    }
}
