<?php

namespace Pinpoint\Conesso\Observer\Newsletter;

/**
 * @SuppressWarnings(PHPMD.CouplingBetweenObjects)
 */
class NewsletterSubscriberSaveAfter implements \Magento\Framework\Event\ObserverInterface
{
    /**
     * @var \Pinpoint\Conesso\Helper\Config
     */
    private $configHelper;

    /**
     * @var \Magento\Framework\App\Response\RedirectInterface
     */
    private $redirect;

    /**
     * @var \Magento\Framework\HTTP\Header
     */
    private $header;

    /**
     * @var \Magento\Framework\Stdlib\DateTime\TimezoneInterface
     */
    private $timezone;

    /**
     * @var \Magento\Framework\App\Request\Http
     */
    private $http;

    /**
     * @var \Pinpoint\Conesso\Model\ConsentFactory
     */
    private $consentFactory;

    /**
     * @var \Pinpoint\Conesso\Model\ResourceModel\Consent
     */
    private $consentResource;

    /**
     * @var \Pinpoint\Conesso\Helper\Data
     */
    private $helper;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    private $storeManager;

    /**
     * @var \Pinpoint\Conesso\Model\ContactFactory
     */
    private $contactFactory;

    /**
     * NewsletterSubscriberSaveAfter constructor.
     * @param \Pinpoint\Conesso\Model\ConsentFactory             $consentFactory
     * @param \Pinpoint\Conesso\Model\ContactFactory             $contactFactory
     * @param \Pinpoint\Conesso\Model\ResourceModel\Consent      $consentResource
     * @param \Pinpoint\Conesso\Helper\Data                      $data
     * @param \Magento\Store\Model\StoreManagerInterface           $storeManagerInterface
     * @param \Magento\Framework\Stdlib\DateTime\TimezoneInterface $timezone
     * @param \Magento\Framework\App\Request\Http                  $http
     * @param \Magento\Framework\HTTP\Header                       $header
     * @param \Magento\Framework\App\Response\RedirectInterface    $redirect
     */
    public function __construct(
        \Pinpoint\Conesso\Model\ConsentFactory $consentFactory,
        \Pinpoint\Conesso\Model\ContactFactory $contactFactory,
        \Pinpoint\Conesso\Model\ResourceModel\Consent $consentResource,
        \Pinpoint\Conesso\Helper\Data $data,
        \Magento\Store\Model\StoreManagerInterface $storeManagerInterface,
        \Magento\Framework\Stdlib\DateTime\TimezoneInterface $timezone,
        \Magento\Framework\App\Request\Http $http,
        \Magento\Framework\HTTP\Header $header,
        \Magento\Framework\App\Response\RedirectInterface $redirect
    ) {
        $this->http  = $http;
        $this->header = $header;
        $this->redirect = $redirect;
        $this->helper = $data;
        $this->configHelper = $this->helper->configHelperFactory->create();
        $this->timezone = $timezone;
        $this->storeManager   = $storeManagerInterface;
        $this->consentFactory = $consentFactory;
        $this->contactFactory = $contactFactory;
        $this->consentResource = $consentResource;
    }

    /**
     * @param \Magento\Framework\Event\Observer $observer
     * @return $this
     */
    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        $subscriber = $observer->getEvent()->getSubscriber();
        $email = $subscriber->getEmail();
        $subscriberStatus = $subscriber->getSubscriberStatus();
        $websiteId = $this->storeManager->getStore($subscriber->getStoreId())
            ->getWebsiteId();

        //If not confirmed or not enabled.
        if ($subscriberStatus == \Magento\Newsletter\Model\Subscriber::STATUS_UNSUBSCRIBED ||
            !$this->helper->isEnabled($websiteId) ||
            !$this->configHelper->isConsentSubscriberEnabled($websiteId)
        ) {
            return $this;
        }

        try {
            $contactEmail = $this->contactFactory->create()
                ->loadByCustomerEmail($email, $websiteId);
            $emailContactId = $contactEmail->getId();
            $consentModel = $this->consentFactory->create();

            $this->consentResource->load(
                $consentModel,
                $emailContactId,
                'email_contact_id'
            );

            //don't update the consent data for guest subscribers or not confrimed
            if (! $consentModel->isObjectNew()) {
                return $this;
            }

            $consentIp = $this->http->getClientIp();
            $consentUrl = $this->redirect->getRefererUrl();
            $consentUserAgent = $this->header->getHttpUserAgent();
            //save the consent data against the contact
            $consentModel->setEmailContactId($emailContactId)
                ->setConsentUrl($consentUrl)
                ->setConsentDatetime(time())
                ->setConsentIp($consentIp)
                ->setConsentUserAgent($consentUserAgent);

            //save contact
            $this->consentResource->save($consentModel);
        } catch (\Exception $e) {
            $this->helper->debug((string)$e, []);
        }

        return $this;
    }
}
