<?php

namespace Pinpoint\Conesso\Observer\Newsletter;

/**
 * Remove contact single delete.
 */
class RemoveContact implements \Magento\Framework\Event\ObserverInterface
{
    /**
     * @var \Pinpoint\Conesso\Helper\Data
     */
    private $helper;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    private $storeManager;

    /**
     * @var \Pinpoint\Conesso\Model\ContactFactory
     */
    private $contactFactory;

    /**
     * @var \Pinpoint\Conesso\Model\ImporterFactory
     */
    private $importerFactory;

    /**
     * @var \Pinpoint\Conesso\Model\ResourceModel\Contact
     */
    private $contactResource;

    /**
     * RemoveContact constructor.
     *
     * @param \Pinpoint\Conesso\Model\ResourceModel\Contact $contactResource
     * @param \Pinpoint\Conesso\Model\ContactFactory        $contactFactory
     * @param \Pinpoint\Conesso\Helper\Data                 $data
     * @param \Magento\Store\Model\StoreManagerInterface      $storeManagerInterface
     * @param \Pinpoint\Conesso\Model\ImporterFactory       $importerFactory
     */
    public function __construct(
        \Pinpoint\Conesso\Model\ResourceModel\Contact $contactResource,
        \Pinpoint\Conesso\Model\ContactFactory $contactFactory,
        \Pinpoint\Conesso\Helper\Data $data,
        \Magento\Store\Model\StoreManagerInterface $storeManagerInterface,
        \Pinpoint\Conesso\Model\ImporterFactory $importerFactory
    ) {

        $this->contactFactory = $contactFactory;
        $this->helper = $data;
        $this->storeManager = $storeManagerInterface;
        $this->importerFactory = $importerFactory;
        $this->contactResource = $contactResource;
    }

    /**
     * Remove contact from account
     *
     * @param \Magento\Framework\Event\Observer $observer
     *
     * @return $this
     */
    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        $subscriber = $observer->getEvent()->getSubscriber();
        $email = $subscriber->getEmail();
        $websiteId = $this->storeManager->getStore($subscriber->getStoreId())
            ->getWebsiteId();
        $apiEnabled = $this->helper->isEnabled($websiteId);

        /*
         * Remove contact.
         */
        if ($apiEnabled) {
            try {
                //register in queue with importer
                $this->importerFactory->create()->registerQueue(
                    \Pinpoint\Conesso\Model\Importer::IMPORT_TYPE_CONTACT,
                    $email,
                    \Pinpoint\Conesso\Model\Importer::MODE_CONTACT_DELETE,
                    $websiteId
                );
                $contactModel = $this->contactFactory->create()
                    ->loadByCustomerEmail($email, $websiteId);
                if ($contactModel->getId()) {
                    //remove contact
                    $this->contactResource->delete($contactModel);
                }
            } catch (\Exception $e) {
                $this->helper->debug((string)$e, []);
            }
        }

        return $this;
    }
}
