<?php

namespace Pinpoint\Conesso\Observer\Sales;

use Magento\Framework\Event\Observer;
use Magento\Framework\Registry;
use Magento\Store\Model\StoreManagerInterface;
use Pinpoint\Conesso\Helper\Data;
use Pinpoint\Conesso\Model\OrderFactory;
use Pinpoint\Conesso\Model\ResourceModel\Order;

/**
 * Class OrderSaveAfter
 * @package Pinpoint\Conesso\Observer\Sales
 */
class OrderSaveAfter implements \Magento\Framework\Event\ObserverInterface
{
    /**
     * @var Order
     */
    private $orderResource;

    /**
     * @var Registry
     */
    private $registry;

    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * @var OrderFactory
     */
    private $emailOrderFactory;

    /**
     * @var Data
     */
    private $helper;

    /**
     * OrderSaveAfter constructor.
     * @param Order                 $orderResource
     * @param OrderFactory          $emailOrderFactory
     * @param Registry              $registry
     * @param StoreManagerInterface $storeManagerInterface
     * @param Data                  $data
     */
    public function __construct(
        Order $orderResource,
        OrderFactory $emailOrderFactory,
        Registry $registry,
        StoreManagerInterface $storeManagerInterface,
        Data $data
    ) {
        $this->orderResource = $orderResource;
        $this->emailOrderFactory      = $emailOrderFactory;
        $this->storeManager           = $storeManagerInterface;
        $this->registry               = $registry;
        $this->helper                 = $data;
    }

    /**
     * @param Observer $observer
     * @return $this|void
     * @throws AlreadyExistsException
     * @throws NoSuchEntityException
     */
    public function execute(Observer $observer)
    {

        $order = $observer->getEvent()->getOrder();
        $status         = $order->getStatus();
        $storeId        = $order->getStoreId();
        $store      = $this->storeManager->getStore($storeId);

        // Check if it already exists
        $emailOrder = $this->emailOrderFactory->create()
            ->loadByOrderId($order->getEntityId(), $order->getQuoteId());

        //$emailOrder->delete();
        return $this;
        //reimport email order
        $emailOrder->setUpdatedAt($order->getUpdatedAt())
            ->setCreatedAt($order->getUpdatedAt())
            ->setStoreId($store->getId())
            ->setOrderStatus($status);

        $emailOrder->setConessoImported(null);

        $isEnabled = $this->helper->isStoreEnabled($store->getId());

        //api not enabled, stop emulation and exit
        if (!$isEnabled) {
            return $this;
        }

        // check for order status change
        $this->handleOrderStatusChange($status, $emailOrder);

        //var_dump($emailOrder->getData());
        //die();
        // set back the current store
        //$emailOrder->save();
        //$emailOrder->save();
        $this->orderResource->save($emailOrder);

        //admin oder when editing the first one is canceled
        $this->registry->unregister('sales_order_status_before');

        return $this;
    }

    /**
     * @param $status
     * @param $emailOrder
     */
    private function handleOrderStatusChange($status, $emailOrder)
    {
        $statusBefore = $this->registry->registry('sales_order_status_before');
        if ($status != $statusBefore) {
            //If order status has changed and order is already imported then set modified to 1
            if ($emailOrder->getConessoImported() == \Pinpoint\Conesso\Model\Contact::EMAIL_CONTACT_IMPORTED) {
                $emailOrder->setModified(\Pinpoint\Conesso\Model\Contact::EMAIL_CONTACT_IMPORTED);
            }
        }
    }
}
