<?php

namespace Pinpoint\Conesso\Observer\Sales;

use Magento\Framework\Event\Observer;
use Magento\Framework\Registry;
use Magento\Sales\Api\Data\OrderInterfaceFactory;
use Magento\Sales\Model\Spi\OrderResourceInterface;

/**
 * Class OrderSaveBefore
 * @package Pinpoint\Conesso\Observer\Sales
 */
class OrderSaveBefore implements \Magento\Framework\Event\ObserverInterface
{
    /**
     * @var Registry
     */
    private $registry;

    /**
     * @var OrderInterfaceFactory
     */
    private $orderFactory;

    /**
     * @var OrderResourceInterface
     */
    private $orderResource;

    /**
     * OrderSaveBefore constructor.
     * @param OrderResourceInterface $orderResource
     * @param OrderInterfaceFactory  $orderFactory
     * @param Registry               $registry
     */
    public function __construct(
        OrderResourceInterface $orderResource,
        OrderInterfaceFactory $orderFactory,
        Registry $registry
    ) {
        $this->orderResource = $orderResource;
        $this->orderFactory = $orderFactory;
        $this->registry     = $registry;
    }

    /**
     * @param Observer $observer
     * @return $this|void
     */
    public function execute(Observer $observer)
    {
        $order = $observer->getEvent()->getOrder();
        //order is new
        if (!$order->getId()) {
            $orderStatus = $order->getStatus();
        } else {
            // the reloaded status
            $reloaded = $this->orderFactory->create();
            $this->orderResource->load($reloaded, $order->getId());
            $orderStatus = $reloaded->getStatus();
        }
        //register the order status before change
        if (is_null($this->registry->registry('sales_order_status_before'))) {
            $this->registry->register(
                'sales_order_status_before',
                $orderStatus
            );
        }

        return $this;
    }
}
