<?php

namespace Pinpoint\Conesso\Setup;

use Magento\Framework\DB\Ddl\Table;
use Magento\Framework\Setup\InstallSchemaInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\SchemaSetupInterface;

/**
 * @codeCoverageIgnore
 */
class InstallSchema implements InstallSchemaInterface
{

    /**
     * InstallSchema constructor.
     */
    public function __construct()
    {
    }

    /**
     * {@inheritdoc}
     * @throws Zend_Db_Exception
     */
    public function install(
        SchemaSetupInterface $setup,
        ModuleContextInterface $context
    ) {
        $installer = $setup;
        $installer->startSetup();

        /**
         * Create Table
         */
        $this->createContactTable($installer);
        $this->createOrderTable($installer);
        $this->createCatalogTable($installer);
        $this->createRuleTable($installer);
        $this->createImporterTable($installer);
        $this->createAbandonedCartTable($installer);

        /**
         * Modify table
         */
        $this->addColumnToAdminUserTable($installer);

        $installer->endSetup();
    }

    /**
     * @param SchemaSetupInterface $installer
     *
     * @return null
     */
    private function createContactTable(SchemaSetupInterface $installer)
    {
        $tableName = $installer->getTable(Schema::CONESSO_CONTACT_TABLE);
        $this->dropTableIfExists($installer, $tableName);

        $contactTable = $installer->getConnection()->newTable($tableName);
        $contactTable = $this->addColumnsToContactTable($contactTable);
        $contactTable = $this->addIndexesToContactTable($installer, $contactTable);

        $contactTable->addForeignKey(
            $installer->getFkName(
                Schema::CONESSO_CONTACT_TABLE,
                'website_id',
                'store_website',
                'website_id'
            ),
            'website_id',
            $installer->getTable('store_website'),
            'website_id',
            Table::ACTION_CASCADE
        );

        $contactTable->setComment('Connector Contacts');
        $installer->getConnection()->createTable($contactTable);
    }

    /**
     * @param $contactTable
     * @return mixed
     */
    private function addColumnsToContactTable($contactTable)
    {
        return $contactTable->addColumn(
            'conesso_contact_id',
            Table::TYPE_INTEGER,
            10,
            [
                'primary' => true,
                'identity' => true,
                'unsigned' => true,
                'nullable' => false
            ],
            'Primary Key'
        )
            ->addColumn(
                'is_guest',
                Table::TYPE_SMALLINT,
                null,
                ['unsigned' => true, 'nullable' => true],
                'Is Guest'
            )
            ->addColumn(
                'contact_id',
                Table::TYPE_TEXT,
                15,
                ['unsigned' => true, 'nullable' => true],
                'Connector Contact ID'
            )
            ->addColumn(
                'customer_id',
                Table::TYPE_INTEGER,
                11,
                ['unsigned' => true, 'nullable' => false],
                'Customer ID'
            )
            ->addColumn(
                'website_id',
                Table::TYPE_SMALLINT,
                5,
                ['unsigned' => true, 'nullable' => false, 'default' => '0'],
                'Website ID'
            )
            ->addColumn(
                'store_id',
                Table::TYPE_SMALLINT,
                5,
                ['unsigned' => true, 'nullable' => false, 'default' => '0'],
                'Store ID'
            )
            ->addColumn(
                'email',
                Table::TYPE_TEXT,
                255,
                ['nullable' => false, 'default' => ''],
                'Customer Email'
            )
            ->addColumn(
                'is_subscriber',
                Table::TYPE_SMALLINT,
                null,
                ['unsigned' => true, 'nullable' => true],
                'Is Subscriber'
            )
            ->addColumn(
                'subscriber_status',
                Table::TYPE_SMALLINT,
                null,
                ['unsigned' => true, 'nullable' => true],
                'Subscriber status'
            )
            ->addColumn(
                'conesso_imported',
                Table::TYPE_SMALLINT,
                null,
                ['unsigned' => true, 'nullable' => true],
                'Is Imported'
            )
            ->addColumn(
                'subscriber_imported',
                Table::TYPE_SMALLINT,
                null,
                ['unsigned' => true, 'nullable' => true],
                'Subscriber Imported'
            )
            ->addColumn(
                'suppressed',
                Table::TYPE_SMALLINT,
                null,
                ['unsigned' => true, 'nullable' => true],
                'Is Suppressed'
            )
            ->addColumn(
                'last_subscribed_at',
                Table::TYPE_TIMESTAMP,
                null,
                [],
                'Last time user subscribed'
            );
    }

    /**
     * @param $installer
     * @param $contactTable
     * @return mixed
     */
    private function addIndexesToContactTable($installer, $contactTable)
    {
        return $contactTable->addIndex(
            $installer->getIdxName(Schema::CONESSO_CONTACT_TABLE, ['conesso_contact_id']),
            ['conesso_contact_id']
        )
            ->addIndex(
                $installer->getIdxName(Schema::CONESSO_CONTACT_TABLE, ['is_guest']),
                ['is_guest']
            )
            ->addIndex(
                $installer->getIdxName(Schema::CONESSO_CONTACT_TABLE, ['customer_id']),
                ['customer_id']
            )
            ->addIndex(
                $installer->getIdxName(Schema::CONESSO_CONTACT_TABLE, ['website_id']),
                ['website_id']
            )
            ->addIndex(
                $installer->getIdxName(Schema::CONESSO_CONTACT_TABLE, ['is_subscriber']),
                ['is_subscriber']
            )
            ->addIndex(
                $installer->getIdxName(Schema::CONESSO_CONTACT_TABLE, ['subscriber_status']),
                ['subscriber_status']
            )
            ->addIndex(
                $installer->getIdxName(Schema::CONESSO_CONTACT_TABLE, ['conesso_imported']),
                ['conesso_imported']
            )
            ->addIndex(
                $installer->getIdxName(Schema::CONESSO_CONTACT_TABLE, ['subscriber_imported']),
                ['subscriber_imported']
            )
            ->addIndex(
                $installer->getIdxName(Schema::CONESSO_CONTACT_TABLE, ['suppressed']),
                ['suppressed']
            )
            ->addIndex(
                $installer->getIdxName(Schema::CONESSO_CONTACT_TABLE, ['email']),
                ['email']
            )
            ->addIndex(
                $installer->getIdxName(Schema::CONESSO_CONTACT_TABLE, ['contact_id']),
                ['contact_id']
            );
    }

    /**
     * @param $installer
     * @throws Zend_Db_Exception
     */
    private function createOrderTable($installer)
    {
        $tableName = $installer->getTable(Schema::CONESSO_ORDER_TABLE);
        $this->dropTableIfExists($installer, $tableName);

        $orderTable = $installer->getConnection()->newTable($tableName);
        $orderTable = $this->addColumnsToOrderTable($orderTable);
        $orderTable = $this->addIndexesToOrderTable($installer, $orderTable);
        $orderTable->addForeignKey(
            $installer->getFkName(
                $installer->getTable(Schema::CONESSO_ORDER_TABLE),
                'store_id',
                'store',
                'store_id'
            ),
            'store_id',
            $installer->getTable('store'),
            'store_id',
            Table::ACTION_CASCADE,
            Table::ACTION_CASCADE
        );

        //Only add foreign key if table exist in default connection
        if ($installer->getConnection()->isTableExists($installer->getTable('sales_order'))) {
            $orderTable->addForeignKey(
                $installer->getFkName(
                    $installer->getTable(Schema::CONESSO_ORDER_TABLE),
                    'order_id',
                    'sales_order',
                    'entity_id'
                ),
                'order_id',
                $installer->getTable('sales_order'),
                'entity_id',
                Table::ACTION_CASCADE,
                Table::ACTION_CASCADE
            );
        }

        $orderTable->setComment('Transactional Order Data');
        $installer->getConnection()->createTable($orderTable);
    }

    /**
     * @param $orderTable
     * @return mixed
     */
    private function addColumnsToOrderTable($orderTable)
    {
        return $orderTable->addColumn(
            'conesso_order_id',
            Table::TYPE_INTEGER,
            null,
            [
                'primary' => true,
                'identity' => true,
                'unsigned' => true,
                'nullable' => false
            ],
            'Primary Key'
        )
        ->addColumn(
            'order_id',
            Table::TYPE_INTEGER,
            null,
            ['unsigned' => true, 'nullable' => false],
            'Order ID'
        )
        ->addColumn(
            'order_status',
            Table::TYPE_TEXT,
            255,
            ['unsigned' => true, 'nullable' => false],
            'Order Status'
        )
        ->addColumn(
            'quote_id',
            Table::TYPE_INTEGER,
            null,
            ['unsigned' => true, 'nullable' => false],
            'Sales Quote ID'
        )
        ->addColumn(
            'store_id',
            Table::TYPE_SMALLINT,
            5,
            ['unsigned' => true, 'nullable' => false, 'default' => '0'],
            'Store ID'
        )
        ->addColumn(
            'conesso_imported',
            Table::TYPE_SMALLINT,
            null,
            ['unsigned' => true, 'nullable' => true],
            'Is Order Imported'
        )
        ->addColumn(
            'modified',
            Table::TYPE_SMALLINT,
            null,
            ['unsigned' => true, 'nullable' => true],
            'Is Order Modified'
        )
        ->addColumn(
            'created_at',
            Table::TYPE_TIMESTAMP,
            null,
            [],
            'Creation Time'
        )
        ->addColumn(
            'updated_at',
            Table::TYPE_TIMESTAMP,
            null,
            [],
            'Update Time'
        );
    }

    /**
     * @param $installer
     * @param $orderTable
     * @return mixed
     */
    private function addIndexesToOrderTable($installer, $orderTable)
    {
        return $orderTable->addIndex(
            $installer->getIdxName(
                $installer->getTable(Schema::CONESSO_ORDER_TABLE),
                ['store_id']
            ),
            ['store_id']
        )
        ->addIndex(
            $installer->getIdxName(
                $installer->getTable(Schema::CONESSO_ORDER_TABLE),
                ['quote_id']
            ),
            ['quote_id']
        )
        ->addIndex(
            $installer->getIdxName(
                $installer->getTable(Schema::CONESSO_ORDER_TABLE),
                ['conesso_imported']
            ),
            ['conesso_imported']
        )
        ->addIndex(
            $installer->getIdxName(
                $installer->getTable(Schema::CONESSO_ORDER_TABLE),
                ['order_status']
            ),
            ['order_status']
        )
        ->addIndex(
            $installer->getIdxName(
                $installer->getTable(Schema::CONESSO_ORDER_TABLE),
                ['modified']
            ),
            ['modified']
        )
        ->addIndex(
            $installer->getIdxName(
                $installer->getTable(Schema::CONESSO_ORDER_TABLE),
                ['updated_at']
            ),
            ['updated_at']
        )
        ->addIndex(
            $installer->getIdxName(
                $installer->getTable(Schema::CONESSO_ORDER_TABLE),
                ['created_at']
            ),
            ['created_at']
        );
    }

    /**
     * @param $installer
     * @throws Zend_Db_Exception
     */
    private function createCatalogTable($installer)
    {
        $tableName = $installer->getTable(Schema::CONESSO_CATALOG_TABLE);
        $this->dropTableIfExists($installer, $tableName);

        $catalogTable = $installer->getConnection()->newTable($tableName);
        $catalogTable = $this->addColumnsToCatalogTable($catalogTable);
        $catalogTable = $this->addIndexesToCatalogTable($installer, $catalogTable);
        $catalogTable->addForeignKey(
            $installer->getFkName(
                Schema::CONESSO_CATALOG_TABLE,
                'product_id',
                'catalog_product_entity',
                'entity_id'
            ),
            'product_id',
            $installer->getTable('catalog_product_entity'),
            'entity_id',
            Table::ACTION_CASCADE
        );

        $catalogTable->setComment('Connector Catalog');
        $installer->getConnection()->createTable($catalogTable);
    }

    /**
     * @param $catalogTable
     * @return mixed
     */
    private function addColumnsToCatalogTable($catalogTable)
    {
        return $catalogTable->addColumn(
            'id',
            Table::TYPE_INTEGER,
            null,
            [
                'primary' => true,
                'identity' => true,
                'unsigned' => true,
                'nullable' => false
            ],
            'Primary Key'
        )
        ->addColumn(
            'product_id',
            Table::TYPE_INTEGER,
            10,
            ['unsigned' => true, 'nullable' => false],
            'Product Id'
        )
        ->addColumn(
            'imported',
            Table::TYPE_SMALLINT,
            null,
            ['unsigned' => true, 'nullable' => true],
            'Product Imported'
        )
        ->addColumn(
            'modified',
            Table::TYPE_SMALLINT,
            null,
            ['unsigned' => true, 'nullable' => true],
            'Product Modified'
        )
        ->addColumn(
            'created_at',
            Table::TYPE_TIMESTAMP,
            null,
            [],
            'Creation Time'
        )
        ->addColumn(
            'updated_at',
            Table::TYPE_TIMESTAMP,
            null,
            [],
            'Update Time'
        );
    }

    /**
     * @param $installer
     * @param $catalogTable
     * @return mixed
     */
    private function addIndexesToCatalogTable($installer, $catalogTable)
    {
        return $catalogTable->addIndex(
            $installer->getIdxName(
                $installer->getTable(Schema::CONESSO_CATALOG_TABLE),
                ['product_id']
            ),
            ['product_id']
        )
        ->addIndex(
            $installer->getIdxName(
                $installer->getTable(Schema::CONESSO_CATALOG_TABLE),
                ['imported']
            ),
            ['imported']
        )
        ->addIndex(
            $installer->getIdxName(
                $installer->getTable(Schema::CONESSO_CATALOG_TABLE),
                ['modified']
            ),
            ['modified']
        )
        ->addIndex(
            $installer->getIdxName(
                $installer->getTable(Schema::CONESSO_CATALOG_TABLE),
                ['created_at']
            ),
            ['created_at']
        )
        ->addIndex(
            $installer->getIdxName(
                $installer->getTable(Schema::CONESSO_CATALOG_TABLE),
                ['updated_at']
            ),
            ['updated_at']
        );
    }

    /**
     * @param $installer
     */
    private function createRuleTable($installer)
    {
        $tableName = $installer->getTable(Schema::CONESSO_RULES_TABLE);
        $this->dropTableIfExists($installer, $tableName);

        $ruleTable = $installer->getConnection()->newTable($tableName);
        $ruleTable = $this->addColumnsToRulesTable($ruleTable);
        $ruleTable->setComment('Connector Rules');
        $installer->getConnection()->createTable($ruleTable);
    }

    /**
     * @param $ruleTable
     * @return mixed
     */
    private function addColumnsToRulesTable($ruleTable)
    {
        return $ruleTable->addColumn(
            'id',
            Table::TYPE_INTEGER,
            null,
            [
                'primary' => true,
                'identity' => true,
                'unsigned' => true,
                'nullable' => false
            ],
            'Primary Key'
        )
        ->addColumn(
            'name',
            Table::TYPE_TEXT,
            255,
            ['nullable' => false, 'default' => ''],
            'Rule Name'
        )
        ->addColumn(
            'website_ids',
            Table::TYPE_TEXT,
            255,
            ['nullable' => false, 'default' => '0'],
            'Website Id'
        )
        ->addColumn(
            'type',
            Table::TYPE_SMALLINT,
            null,
            ['nullable' => false, 'default' => 0],
            'Rule Type'
        )
        ->addColumn(
            'status',
            Table::TYPE_SMALLINT,
            null,
            ['nullable' => false, 'default' => 0],
            'Status'
        )
        ->addColumn(
            'combination',
            Table::TYPE_SMALLINT,
            null,
            ['nullable' => false, 'default' => '1'],
            'Rule Condition'
        )
        ->addColumn(
            'condition',
            Table::TYPE_BLOB,
            null,
            ['nullable' => false, 'default' => ''],
            'Rule Condition'
        )
        ->addColumn(
            'created_at',
            Table::TYPE_TIMESTAMP,
            null,
            [],
            'Creation Time'
        )
        ->addColumn(
            'updated_at',
            Table::TYPE_TIMESTAMP,
            null,
            [],
            'Update Time'
        );
    }

    /**
     * @param $installer
     */
    private function createImporterTable($installer)
    {
        $tableName = $installer->getTable(Schema::CONESSO_IMPORTER_TABLE);
        $this->dropTableIfExists($installer, $tableName);

        $importerTable = $installer->getConnection()->newTable($tableName);
        $importerTable = $this->addColumnsToImporterTable($importerTable);
        $importerTable = $this->addIndexesToImporterTable($installer, $importerTable);
        $importerTable ->setComment('Email Importer');
        $installer->getConnection()->createTable($importerTable);
    }

    /**
     * @param $importerTable
     * @return mixed
     */
    private function addColumnsToImporterTable($importerTable)
    {
        return $importerTable->addColumn(
            'id',
            Table::TYPE_INTEGER,
            null,
            [
                'primary' => true,
                'identity' => true,
                'unsigned' => true,
                'nullable' => false
            ],
            'Primary Key'
        )
        ->addColumn(
            'import_type',
            Table::TYPE_TEXT,
            255,
            ['nullable' => false, 'default' => ''],
            'Import Type'
        )
        ->addColumn(
            'website_id',
            Table::TYPE_SMALLINT,
            5,
            ['nullable' => false, 'default' => '0'],
            'Website Id'
        )
        ->addColumn(
            'import_status',
            Table::TYPE_SMALLINT,
            null,
            ['nullable' => false, 'default' => 0],
            'Import Status'
        )
        ->addColumn(
            'import_id',
            Table::TYPE_TEXT,
            255,
            ['nullable' => false, 'default' => ''],
            'Import Id'
        )
        ->addColumn(
            'import_data',
            Table::TYPE_BLOB,
            '2M',
            ['nullable' => false, 'default' => ''],
            'Import Data'
        )
        ->addColumn(
            'import_mode',
            Table::TYPE_TEXT,
            255,
            ['nullable' => false, 'default' => ''],
            'Import Mode'
        )
        ->addColumn(
            'import_file',
            Table::TYPE_TEXT,
            null,
            ['nullable' => false, 'default' => ''],
            'Import File'
        )
        ->addColumn(
            'message',
            Table::TYPE_TEXT,
            255,
            ['nullable' => false, 'default' => ''],
            'Error Message'
        )
        ->addColumn(
            'created_at',
            Table::TYPE_TIMESTAMP,
            null,
            [],
            'Creation Time'
        )
        ->addColumn(
            'updated_at',
            Table::TYPE_TIMESTAMP,
            null,
            [],
            'Update Time'
        )
        ->addColumn(
            'import_started',
            Table::TYPE_TIMESTAMP,
            null,
            [],
            'Import Started'
        )
        ->addColumn(
            'import_finished',
            Table::TYPE_TIMESTAMP,
            null,
            [],
            'Import Finished'
        );
    }

    /**
     * @param $installer
     * @param $importerTable
     * @return mixed
     */
    private function addIndexesToImporterTable($installer, $importerTable)
    {
        return $importerTable->addIndex(
            $installer->getIdxName(
                $installer->getTable(Schema::CONESSO_IMPORTER_TABLE),
                ['import_type']
            ),
            ['import_type']
        )
        ->addIndex(
            $installer->getIdxName(
                $installer->getTable(Schema::CONESSO_IMPORTER_TABLE),
                ['website_id']
            ),
            ['website_id']
        )
        ->addIndex(
            $installer->getIdxName(
                $installer->getTable(Schema::CONESSO_IMPORTER_TABLE),
                ['import_status']
            ),
            ['import_status']
        )
        ->addIndex(
            $installer->getIdxName(
                $installer->getTable(Schema::CONESSO_IMPORTER_TABLE),
                ['import_mode']
            ),
            ['import_mode']
        )
        ->addIndex(
            $installer->getIdxName(
                $installer->getTable(Schema::CONESSO_IMPORTER_TABLE),
                ['created_at']
            ),
            ['created_at']
        )
        ->addIndex(
            $installer->getIdxName(
                $installer->getTable(Schema::CONESSO_IMPORTER_TABLE),
                ['updated_at']
            ),
            ['updated_at']
        )
        ->addIndex(
            $installer->getIdxName(
                $installer->getTable(Schema::CONESSO_IMPORTER_TABLE),
                ['import_id']
            ),
            ['import_id']
        )
        ->addIndex(
            $installer->getIdxName(
                $installer->getTable(Schema::CONESSO_IMPORTER_TABLE),
                ['import_started']
            ),
            ['import_started']
        )
        ->addIndex(
            $installer->getIdxName(
                $installer->getTable(Schema::CONESSO_IMPORTER_TABLE),
                ['import_finished']
            ),
            ['import_finished']
        );
    }

    /**
     * @param $installer
     */
    private function addColumnToAdminUserTable($installer)
    {
        $installer->getConnection()->addColumn(
            $installer->getTable('admin_user'),
            'refresh_token',
            [
            'type' => Table::TYPE_TEXT,
            'length' => 256,
            'nullable' => true,
            'default' => null,
            'comment' => 'Email connector refresh token',
            ]
        );
    }

    /**
     * @param $installer
     */
    public function createAbandonedCartTable($installer)
    {
        $tableName = $installer->getTable(Schema::CONESSO_ABANDONED_CART_TABLE);
        $this->dropTableIfExists($installer, $tableName);

        $abandonedCartTable = $installer->getConnection()->newTable($installer->getTable($tableName));
        $abandonedCartTable = $this->addColumnForAbandonedCartTable($abandonedCartTable);
        $abandonedCartTable = $this->addIndexKeyForAbandonedCarts($installer, $abandonedCartTable);
        $abandonedCartTable->setComment('Abandoned Carts Table');
        $installer->getConnection()->createTable($abandonedCartTable);
    }

    /**
     * @param $table
     * @return mixed
     */
    private function addColumnForAbandonedCartTable($table)
    {
        return $table->addColumn(
            'id',
            Table::TYPE_INTEGER,
            null,
            [
                'primary' => true,
                'identity' => true,
                'unsigned' => true,
                'nullable' => false
            ],
            'Primary Key'
        )
            ->addColumn(
                'quote_id',
                Table::TYPE_INTEGER,
                null,
                ['unsigned' => true, 'nullable' => true],
                'Quote Id'
            )
            ->addColumn(
                'store_id',
                Table::TYPE_SMALLINT,
                10,
                ['unsigned' => true, 'nullable' => true],
                'Store Id'
            )
            ->addColumn(
                'customer_id',
                Table::TYPE_INTEGER,
                10,
                ['unsigned' => true, 'nullable' => true, 'default' => null],
                'Customer ID'
            )
            ->addColumn(
                'email',
                Table::TYPE_TEXT,
                255,
                ['nullable' => false, 'default' => ''],
                'Email'
            )
            ->addColumn(
                'status',
                Table::TYPE_TEXT,
                255,
                ['nullable' => false, 'default' => ''],
                'Contact Status'
            )
            ->addColumn(
                'is_active',
                Table::TYPE_SMALLINT,
                5,
                ['unsigned' => true, 'nullable' => false, 'default' => '1'],
                'Quote Active'
            )
            ->addColumn(
                'quote_updated_at',
                Table::TYPE_TIMESTAMP,
                null,
                [],
                'Quote updated at'
            )
            ->addColumn(
                'abandoned_cart_number',
                Table::TYPE_SMALLINT,
                null,
                ['unsigned' => true, 'nullable' => false, 'default' => 0],
                'Abandoned Cart number'
            )
            ->addColumn(
                'items_count',
                Table::TYPE_SMALLINT,
                null,
                ['unsigned' => true, 'nullable' => true, 'default' => 0],
                'Quote items count'
            )
            ->addColumn(
                'items_ids',
                Table::TYPE_TEXT,
                255,
                ['unsigned' => true, 'nullable' => true],
                'Quote item ids'
            )
            ->addColumn(
                'created_at',
                Table::TYPE_TIMESTAMP,
                null,
                [],
                'Created At'
            )
            ->addColumn(
                'updated_at',
                Table::TYPE_TIMESTAMP,
                null,
                [],
                'Updated at'
            );
    }

    /**
     * @param $installer
     * @param $abandonedCartTable
     * @return mixed
     */
    private function addIndexKeyForAbandonedCarts($installer, $abandonedCartTable)
    {
        return $abandonedCartTable->addIndex(
            $installer->getIdxName(Schema::CONESSO_ABANDONED_CART_TABLE, ['quote_id']),
            ['quote_id']
        )
            ->addIndex(
                $installer->getIdxName(Schema::CONESSO_ABANDONED_CART_TABLE, ['store_id']),
                ['store_id']
            )
            ->addIndex(
                $installer->getIdxName(Schema::CONESSO_ABANDONED_CART_TABLE, ['customer_id']),
                ['customer_id']
            )
            ->addIndex(
                $installer->getIdxName(Schema::CONESSO_ABANDONED_CART_TABLE, ['email']),
                ['email']
            );
    }

    /**
     * @param $installer
     * @param $table
     */
    private function dropTableIfExists($installer, $table)
    {
        if ($installer->getConnection()->isTableExists($installer->getTable($table))) {
            $installer->getConnection()->dropTable(
                $installer->getTable($table)
            );
        }
    }
}
